
/*
		---------------------------------------
		PowerTerm (c) 1992-94  Byron D Copeland
			All Rights Reserved.
		---------------------------------------

*/

// Communications control module

#include <stdio.h>
#include <dos.h>
#include <alloc.h>
#include <signal.h>
#include "serial.h"
#define FALSE           0
#define TRUE            (!FALSE)
#define NOERROR         0       // No error
#define BUFOVFL         1       // Buffer overflowed
#define ESC             0x1B    // ASCII Escape character
#define ASCII           0x007F  // Mask ASCII characters
#define SBUFSIZ         0x4000  // Serial buffer size
#define XON             0x11
#define XOFF            0x13
#define CARRIERLOST     100

static int              CHOLD=FALSE;
int                     COM_WAIT;
int                     TRANS_CLEAR;
int                     FIFO;
int                     CARRIERON;
int                     TXEN=FALSE;
int                     TFIRST=TRUE;
int                     i;
unsigned int            IRQMask;
unsigned int            old_mask;
unsigned int            old_ier;
unsigned char           iir;
unsigned char           lsr;
unsigned char           msr;
unsigned char           ierval;
extern int              SError;
extern int              ONLINE;
extern unsigned int     portbase;
extern int              tstat;
extern unsigned int     port;
extern unsigned long    speed;
extern int              parity;
extern int              bits;
extern int              stopbits;
extern int              LastAttr, VTc, VTr;
extern char             *TempBuf;
static unsigned int     startbuf;
static unsigned int     endbuf;
static unsigned int     ostartbuf;
static unsigned int     oendbuf;
int                     bytes;
int                     tx_running;

extern struct PORTS     {
	unsigned int            portnum;                // COM1 - COM8
	unsigned int            address;                //
	unsigned char           irqmask;                // IRQ0 - IRQ7
	unsigned int            irqline;                // 0 - 7
	unsigned int            intnum;                 // 0x00 - 0xFF
	unsigned char           handshake;
} PortDefs[8];

static unsigned char *ccbuf;
static unsigned char *outbuf;

typedef void interrupt far (*intvect)( void );

intvect         oldvects[2];

void    flushinput( void );
int     RxSpaceUsed( void );
int     check_uart( void );
void    SendCommBreak( void );
void    jmp_start( void );

/* SERIAL.C: An interrupt driven serial port buffer

	This program bypasses the less-than-adequate PC BIOS
	communications routines and installs a serial interrupt
	handler. Direct access to PC hardware allows the program to
	run at faster baud rates and eliminates the need for
	the main program to continuously poll the serial port for
	data; thus implementing background communications. Data that
	enters the serial port is stored in a circular buffer.

	* Compile this program with Test Stack Overflow OFF.
*/

/*-------------------------------------------------------------------
 com_int - Handle communications interrupts and put them in ccbuf.

 Supports 16550A buffered UARTS, which, with an USR HST and a clear
 night, can result in 1740 CPS Rates with DTE locked at 38400 baud!

--------------------------------------------------------------------*/

void interrupt far com_int( void ) {

	int     c;
	int     bytecnt;

	switch( iir = inportb( portbase + IIR ) & 0x07 )        {

		case F_RMST:            // Modem Status

			msr = inportb( portbase + MSR );
			if ( msr & DeltaDCD )   {                       // DCD has changed state
				if ( msr & DCD )
					CARRIERON = TRUE;
				else
					SError = CARRIERLOST;
			}
			break;

		case F_RLST:            // Line Status

			lsr = inportb( portbase + LSR );
			c = inportb( portbase + RXR );
			break;

		case F_RDAV:

			lsr = inportb( portbase + LSR );
			c = inportb( portbase + MCR ) & ~RTS;
			outportb( portbase + MCR, c );
			while ( inportb( portbase + LSR ) & RCVRDY )    {
				ccbuf[endbuf++] = inportb( portbase + RXR );
				endbuf &= ( SBUFSIZ - 1 );

				// Check Flow Control and
				// halt comm if near overrun

				if ( !CHOLD )   {
					if ( ( ( endbuf + 512 ) & SBUFSIZ - 1 ) == startbuf )   {

						// Empty rest of FIFO if any.

						while ( inportb( portbase + LSR ) & RCVRDY )    {
							c = inportb( portbase + RXR );
							ccbuf[endbuf++] = c;
							endbuf &= ( SBUFSIZ - 1 );
						}
						CHOLD = TRUE;
						break;
					}
					else    {
						c = inportb( portbase + MCR ) | RTS;
						outportb( portbase + MCR, c );
					}
				}
			}       /* End while */
			break;

/*              case F_IIR_THRE:

			if ( ostartbuf == oendbuf )     {
				inportb( portbase + IIR );
				tx_running = FALSE;
				break;
			}
			lsr = inportb( portbase + LSR );
			bytecnt = 0;
			if ( lsr & XMTRDY )     {
				if (( !FIFO ) && ( ostartbuf != oendbuf ) )     {
					if ( inportb( portbase + MSR ) & CTS )  {
						c = outbuf[ostartbuf++];
						outportb( portbase + TXR, c );
						ostartbuf %= SBUFSIZ;
					}
					break;
				}
				else
				while ( inportb( portbase + MSR ) & CTS )       {
					if (( bytecnt++ < 15 ) && ( ostartbuf != oendbuf ))     {
						c = outbuf[ostartbuf++];
						outportb( portbase + TXR, c );
						ostartbuf %= SBUFSIZ;
					}
					else    {
						break;
					}
				}
			}
			else    {
				inportb( portbase + IIR );
				break;
			}
			break;  */

		default:
			break;
	}

	// Signal end of hardware interrupt and allow others.

	outportb( portbase + IER, 0 );
	outportb( portbase + IER, IER_VAL2 );
	outportb( ICR, EOI );
	return;
}

/*-------------------------------------------------------------------
	SerialOut - Output a character to the serial port
-------------------------------------------------------------------*/

int SerialOut( int x )
{
	static long int timeout = 0x0000FFFFL;
	outportb( portbase + MCR, MC_INT | DTR | RTS );

	// Wait for Clear To Send from modem

	while ( ( inportb( portbase + MSR ) & CTS ) == 0 )
		if ( !( --timeout ) )
			return( -1 );

	timeout = 0x0000FFFFL;

	// Wait for transmitter to clear

	while ( ( inportb( portbase + LSR ) & XMTRDY ) == 0 )
		if ( !( --timeout ) )
			return( -1 );
//        disable();
	outportb( portbase + TXR, (unsigned char)x );
//        enable();
	return( 0 );
}


void jmp_start( void )
{
	static long int timeout = 0x0000FFFFL;
	int     c;

	disable();
	if ( ( !tx_running ) && ( ostartbuf != oendbuf ) )      {

		outportb( portbase + MCR, MC_INT | DTR | RTS );

		// Wait for Clear To Send from modem

		while ( ( inportb( portbase + MSR ) & CTS ) == 0 )
			if ( !( --timeout ) )
				return;

		timeout = 0x0000FFFFL;

		// Wait for transmitter to clear

		while ( ( inportb( portbase + LSR ) & XMTRDY ) == 0 )
			if ( !( --timeout ) )
				return;

		c = outbuf[ostartbuf++];
		outportb( portbase + TXR, c );
		ostartbuf %= SBUFSIZ;
		tx_running = TRUE;
	}
	else
		tx_running = FALSE;
	enable();
	return;
}

/*-------------------------------------------------------------------
	 FSerialString - Output a string to the serial port FIFO Mode
--------------------------------------------------------------------*/

void FSerialString( register unsigned char *string )
{
	while ( *string )
		SerialOut( *string++ );
}

/*-------------------------------------------------------------------
	 SerialString - Output a string to the serial port
--------------------------------------------------------------------*/

void SerialString( unsigned char * )
{
	while ( *string )       {
		while( SerialOut( *string ) == -1 );
		string++;
	}
}


/*-------------------------------------------------------------------
	 getccb - This routine returns the current value in the buffer
-------------------------------------------------------------------*/

int getccb( void )
{
	register int res;
	register int c;

	if ( endbuf == startbuf )
		return( -1 );
	res = ( int ) ccbuf[startbuf++];
	startbuf %= SBUFSIZ;

	// Make sure we restart comm if we halted it.

	if ( CHOLD )    {
		if ( ( ( endbuf + 2048 ) & SBUFSIZ - 1 ) == startbuf )  {
			c = inportb( portbase + MCR ) | RTS;
			outportb( portbase + MCR, c );
			CHOLD = FALSE;
		}
	}
	return( res );
}

/*-------------------------------------------------------------------
	 setvects - Install our functions to handle communications
-------------------------------------------------------------------*/

void setvects( void )
{
	disable();
	oldvects[0] = ( intvect ) getvect(PortDefs[port].intnum );
	setvect( PortDefs[port].intnum, ( intvect )com_int );
	enable();

}

/*-------------------------------------------------------------------
	 resvects - Uninstall our vectors before exiting the program
-------------------------------------------------------------------*/

void resvects( void )
{
	disable();
	setvect( PortDefs[port].intnum, ( intvect )oldvects[0] );
	enable();
}

/*-------------------------------------------------------------------
	 i_enable - Turn on communications interrupts
-------------------------------------------------------------------*/

void i_enable( unsigned int pnum )
{
	int     c;
	int     m;
	int     mask;

	// Get IMR value for this port

	IRQMask = PortDefs[pnum].irqmask;
//      fprintf( stderr, "\n\n\n\n\n\n\n\nIRQMask = %x\n", IRQMask );
//      m = inportb( IMR ) & IRQMask;
	m = inportb( IMR );
	mask = 1 << ( PortDefs[pnum].irqline % 8 );

	disable();

	// Allow DATA, MSR & LSR  interrupts.

	outportb( portbase + IER, IER_VAL2 );

	// Allow card to send interrupts and bring up DTR & RTS.

	c = inportb( portbase + MCR ) | MC_INT | DTR | RTS;
	outportb( portbase + MCR, c );

	// Set up Mask Register

	outportb( IMR, m & ~mask );
	enable();
//      fprintf( stderr, "portbase = %x\n", portbase );
}

/*-------------------------------------------------------------------
		i_disable - Turn off communications interrupts
-------------------------------------------------------------------*/

void i_disable( void )
{
	int c;

	disable();

	c = inportb( IMR ) | ~IRQMask;

	outportb( IMR, c );
	c = inportb( portbase + MCR ) & ~MC_INT;
	outportb( portbase + MCR, c );

	outportb( portbase + MCR, 0 );

	// Disable Fifo Buffers

	if ( FIFO )     {
		outportb( portbase + FCR, 0x00 );
	}
	enable();
}

/*-------------------------------------------------------------------
	 comm_on - Tell modem that we're ready to go
-------------------------------------------------------------------*/

void comm_on( void )
{
	int c;
	unsigned int pnum;

	pnum = PortDefs[port].portnum;
	i_enable( pnum );

	// Clear regs

	inportb( portbase + RXR );
	inportb( portbase + LSR );
	inportb( portbase + MSR );


//      c = inportb( portbase + MCR ) | DTR | RTS;
//      outportb( portbase + MCR, c );

}

/*-------------------------------------------------------------------
	 comm_off - Go off-line
-------------------------------------------------------------------*/

void comm_off( void )
{
	i_disable();
	outportb( portbase + MCR, 0 );

}

/*-------------------------------------------------------------------
	 initserial -
-------------------------------------------------------------------*/

void initserial( void )
{
	setvects();
	comm_on();
}

/*-------------------------------------------------------------------
	 closeserial -
-------------------------------------------------------------------*/

void closeserial( void )
{
	outportb( portbase + IER, old_ier );
	outportb( IMR, old_mask );

	comm_off();
	resvects();

}

int check_uart( void )
{
	int     c;

	// check for presence of port

	outportb(portbase + LCR, 0x33);         // Write test Value
	if (inportb(portbase + LCR) != 0x33)
		return(-1);                     // Oops! No Port here.

//      outportb( portbase + FCR, CLR_FIFO );
//      c = inportb( portbase + IIR );
//      if ( ( c & 0xf8 ) != 0 )
//              return( -1 );

	// Check for presence of FIFO

	outportb( portbase + FCR,  FIFO_EN );

	// Setup Fifo Flag

	if ( ( inportb ( portbase + IIR ) & 0xC0 ) != 0xC0 )    {
		FIFO = FALSE;
		outportb( portbase + FCR, 0 );
	}
	else    {
		outportb( portbase + FCR, CLR_FIFO );
		FIFO = TRUE;
	}
	return( 0 );
}


/*-------------------------------------------------------------------
	 SetPort - Set the port number to use and make sure that
		   the port number is a valid one.
-------------------------------------------------------------------*/

int SetPort( unsigned int Port )
{
	unsigned int pnum;

	if ( check_uart() )
		return( -1 );
	return (0);
}

/*-------------------------------------------------------------------
	 SetSpeed - This routine sets the speed; will accept funny baud
	 rates. Setting the speed requires that the DLAB be set
	 on.
-------------------------------------------------------------------*/

int SetSpeed( unsigned long Speed )
{
	int     c;
	int     divisor;

	if ( Speed == 0L )            // Avoid divide by zero
		return( -1 );
	else
		divisor = ( int ) ( 115200L / ( unsigned long ) Speed );

	if ( portbase == 0 )
		return( -1 );
	disable();
	c = inportb( portbase + LCR );
	outportb( portbase + LCR, ( c | 0x80 ) ); // Set DLAB
	outportb( portbase + DLL, ( divisor & 0x00FF ) );
	outportb( portbase + DLH, ( ( divisor >> 8 ) & 0x00FF ) );
	outportb( portbase + LCR, c );          // Reset DLAB
	enable();
	return( 0 );
}

/*-------------------------------------------------------------------
	 SetOthers - Set other communications parameters
-------------------------------------------------------------------*/

int SetOthers( int Parity, int Bits, int StopBit )
{
	int setting;

	if ( portbase == 0 )
		return ( -1 );
	if ( Bits < 5 || Bits > 8 )
		return ( -1 );
	if ( StopBit != 1 && StopBit != 2 )
		return ( -1 );
	if ( Parity != NO_PARITY && Parity != ODD_PARITY && Parity != EVEN_PARITY
			&& Parity != MARK_PARITY && Parity != SPACE_PARITY )
		return ( -1 );

	setting = Bits - 5;
	setting |= ( ( StopBit == 1 ) ? 0x00 : 0x04 );
	setting |= Parity;

	outportb( portbase + LCR, setting );
	return( 0 );

}

/*-------------------------------------------------------------------
	 SetSerial - Set up the port
-------------------------------------------------------------------*/

int SetSerial( unsigned int Port, unsigned long Speed,
		int Parity, int Bits, int StopBit )
{
	int     c;
	old_mask = inportb( IMR );
	old_ier = inportb( portbase + IER );

	FIFO = FALSE;
	CHOLD=FALSE;
	COM_WAIT = FALSE;
	TFIRST=TRUE;
	TRANS_CLEAR = TRUE;
	tx_running = FALSE;
	endbuf = startbuf = ostartbuf = oendbuf = 0;
	portbase = PortDefs[Port].address;
	if ( SetPort( Port ) )  {
		resvects();
		return( -1 );
	}
	if ( SetSpeed( Speed ) )        {
		resvects();
		return ( -1 );
	}
	if ( SetOthers( Parity, Bits, StopBit ) )       {
		resvects();
		return ( -1 );
	}
	return( 0 );
}


int InitCommMem( void )
{
	if ( ( ccbuf = ( unsigned char * )malloc( 1 + SBUFSIZ*sizeof( char ) ) ) == NULL )
		return( -1 );
	setstrng( ccbuf, '\0', SBUFSIZ - 1 );
/*      if ( ( outbuf = ( unsigned char * )malloc( 1 + SBUFSIZ*sizeof( char ) ) ) == NULL )
		return( -1 );
	setstrng( outbuf, '\0', SBUFSIZ - 1 );*/
	return( 0 );
}


void CloseCommMem( void )
{
	if ( ccbuf )
		free( ccbuf );
/*      if ( outbuf )
		free( outbuf );*/
}


/*------------------------------------------------------------------------
 *  f l u s h i n p u t
 *-----------------------------------------------------------------------*/

void flushinput( void )
{

	do {

		// Nothing

	} while ( ( getccb() ) != -1 );
}


int ChangeSpeed( unsigned long Speed )
{

	int     c;
	int     divisor;

	if ( Speed == 0L )            // Avoid divide by zero
		return ( -1 );
	else
		divisor = ( int ) ( 115200L / ( unsigned long ) Speed );

	if ( portbase == 0 )
		return ( -1 );

	disable();
	c = inportb( portbase + LCR );
	outportb( portbase + LCR, ( c | 0x80 ) ); // Set DLAB

	outportb( portbase + DLL, ( divisor & 0x00FF ) );
	outportb( portbase + DLH, ( ( divisor >> 8 ) & 0x00FF ) );
	outportb( portbase + LCR, c );          // Reset DLAB

	enable();

	return( 0 );
}

/*-------------------------------------------------------------------
	 ChangeOthers - Set other communications parameters
-------------------------------------------------------------------*/

int ChangeOthers( int Parity, int Bits, int StopBit )
{
	int setting;

	if ( portbase == 0 )
		return( -1 );
	if ( Bits < 5 || Bits > 8 )
		return ( -1 );
	if ( StopBit != 1 && StopBit != 2 )
		return ( -1 );
	if ( Parity != NO_PARITY && Parity != ODD_PARITY && Parity != EVEN_PARITY
			&& Parity != MARK_PARITY && Parity != SPACE_PARITY )
		return( -1 );

	setting = Bits - 5;
	setting |= ( ( StopBit == 1 ) ? 0x00 : 0x04 );
	setting |= Parity;

	disable();

	outportb( portbase + LCR, setting );

	enable();
	return( 0 );

}




int     RxSpaceUsed( void )
{

	if ( startbuf == endbuf )
		return( 0 );
	return( 1 );
}



void SendCommBreak( void )
{

	long            timer;
	int                     c;

	timer = ReadTimeMills() + 300L;
	c = inportb( portbase + LCR ) | LCR_BRK;
	outportb( portbase + LCR, c );
	while ( ReadTimeMills() < timer );
	c = inportb( portbase + LCR ) & ~LCR_BRK;
	outportb( portbase + LCR, c );
}





