
/*
		---------------------------------------
		PowerTerm (c) 1992-94  Byron D Copeland
			 All Rights Reserved.
		---------------------------------------

*/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <alloc.h>
#include <time.h>
#include <mem.h>
#include <dos.h>
#include <dir.h>
#include <io.h>
#include <fcntl.h>
#include <math.h>
#include <signal.h>
#include <setjmp.h>
#include <process.h>
#include <share.h>
#include <sys\types.h>
#include <sys\stat.h>

#include "fsvlib.h"
#include "serial.h"

#define MAINPRG

#include "pckeys.h"
#include "ipcvars.h"
#include "mouse.h"

FILE 		*newhandle;
int 		shandle;

/*------------------------------------------------------------------------
	This is the main module.  May be cluttered, but you can try to
	do it yourself better.
-------------------------------------------------------------------------*/

int main( void )
{
	int		QUIT;
	unsigned int	RowSave;
	unsigned int	ColSave;

	FILE		*MPF;
	FILE		*PTF;

	int			i;
	int			ch;
	int			key;
	int			MRowSave;

	union REGS	xr;

	// Various function pointers.

	typedef void ( * PULLDOWNS ) ();	// Pointer for main menus
	typedef void ( * PROCS ) ();		// Pointer for main functions

	//  Pointers to the functions that will display each pull-down
	//  menu when selected.

	static PULLDOWNS MenuOps[5] =  {
		DisplayM10,DisplayM20,DisplayM30,DisplayM40,LightUpQuit
	};

	//  Pointers to the functions within each pull-down menu.
	//  Accessed by [MenOpIdx][MenPosIdx] combination.
	//  NoOp() is used as a filler function and does nothing.

	static PROCS	Process[6][8] =		{
		&SysInfo,&RegistrationInfo,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,
		&TerminalOps,&HostModeOps,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,
		&DisplayM30A,&ComPortSetup,&PTDirSetup,&EditMacroKeys,&NoOp,
		&NoOp, &NoOp, &SaveModemDefaults,
		&dosShell,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,
		&SetQuitOn,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,
		&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,NoOp
	};

	//  Call to initialize window variables
	//  and save the current screen.
	//  MUST be done to set up correct video operations.

	if ( Init_FSVLib() != 0 )
		exit( 5 );

	//  Save the current cursor position.

	xr.h.ah = 0x03;
	xr.h.bh = 0x00;
	int86( VIDEOINT, &xr, &xr );
	RowSave = xr.h.dh;
	ColSave = xr.h.dl;

	// Allocate memory for buffers and structures

	if ( AllocateMemory() != 0 )	{
		printf( "Not enough MEMORY for PowerTerm Operations!\n" );
		exit( 1 );
	}
	if ( InitCommMem() != 0 )	{
		printf( "Not enough Memory for PowerTerm Buffers!\n" );
		exit( 1 );
	}
	if ( ( PTEnviron = getenv( "POWERTERM" ) ) == NULL )	{
		printf( "No PowerTerm Environment.\n" );
		printf( "Type SET POWERTERM=drive:\[directory]\n" );
		exit( 1 );
	}
	if ( PTEnviron[ strlen( PTEnviron ) - 1] == '\\' )
		PTEnviron[ strlen( PTEnviron ) - 1] = '\0';

	if ( readSetupFiles() != 0 )	{
		printf( "Error reading setup files.\n" );
		exit( 2 );
	}
	ReadMenuFiles();
	CursorOff();
	fsfill( 1, 1, VidRows, VidCols, '', LightGray+BlackBG );

	// Check for mouse support

	MouseInstalled = TurnOnMouse();
	if ( MouseInstalled )	{
		ExplodeWindow( 12, 26, 3, 30, LightCyan+BlueBG,
			LightCyan+BlueBG, NoShadow, Frame1 );
		fswriteC( "Mouse Support Enabled!", 13, VidCols,
			LightCyan+BlueBG );
		alarm( 2 );
		while( ( !kbhit() ) && ( !( M_event & 0x02 ) ) &&
			( timeleft > 0) );
		alarm( 0 );
		CloseWindow();
	}

	// Initialize emulation pointers.

	Term[0].pointer = &ProcessTTY;
	Term[1].pointer = &ProcessVT52;
	Term[2].pointer = &ProcessVT102;
	Term[3].pointer = &ProcessANSI;
	Term[4].pointer = &ProcessAVATAR;
	Term[5].pointer = &ProcessTK4107C;
	Term[6].pointer = &ProcessVip7801;
	Term[7].pointer = &ProcessI3101;

	Fkey[0].pointer = &VT102SKeys;
	Fkey[1].pointer = &VT102SKeys;
	Fkey[2].pointer = &VT102SKeys;
	Fkey[3].pointer = &ANSIKeys;
	Fkey[4].pointer = &VT102SKeys;
	Fkey[5].pointer = &VT102SKeys;
	Fkey[6].pointer = &VIP7801SKeys;
	Fkey[7].pointer = &I3101SKeys;

	Tkey[0].pointer = &VT102Tab;
	Tkey[1].pointer = &VT102Tab;
	Tkey[2].pointer = &VT102Tab;
	Tkey[3].pointer = &ANSITab;
	Tkey[4].pointer = &VT102Tab;
	Tkey[5].pointer = &Tek4107Tab;
	Tkey[6].pointer = &Vip7801Tab;
	Tkey[7].pointer = &I3101Tab;

	// Initialize variables

	VIPEMU = 6;	// Start with ANSI
	VIP_STS_LINE_IN_USE = FALSE;
	MainMax = 4;
	DEFCHG = FALSE;
	NUMLOCK= FALSE;
	STATLINE = TRUE;
	UPPERONLY = FALSE;
	FnChk = FALSE;

	MenPos[0].Row = 4;
	MenPos[0].Col = 2;
	MenPos[0].RowMin = 4;
	MenPos[0].MaxIdx = 1;
	MenPos[0].RowMax = 5;
	MenPos[0].MaxRows = 2;
	MenPos[0].MaxCol = 19;

	MenPos[1].Row = 4;
	MenPos[1].Col = 14;
	MenPos[1].RowMin = 4;
	MenPos[1].MaxIdx = 1;
	MenPos[1].RowMax = 5;
	MenPos[1].MaxRows = 2;
	MenPos[1].MaxCol = 28;

	MenPos[2].Row = 4;
	MenPos[2].Col = 28;
	MenPos[2].RowMin = 4;
	MenPos[2].MaxIdx = 7;
	MenPos[2].RowMax = 11;
	MenPos[2].MaxRows = 8;
	MenPos[2].MaxCol = 49;

	MenPos[3].Row = 4;
	MenPos[3].Col = 40;
	MenPos[3].RowMin = 4;
	MenPos[3].MaxIdx = 1;
	MenPos[3].RowMax = 5;
	MenPos[3].MaxRows = 2;
	MenPos[3].MaxCol = 56;

	MSPos[2].Row = 4;
	MSPos[2].Col = 52;
	MSPos[2].RowMin = 4;
	MSPos[2].MaxIdx = 2;
	MSPos[2].RowMax = 6;
	MSPos[2].MaxRows = 3;
	MSPos[2].MaxCol = 72;

	M[0] = M10;
	M[1] = M20;
	M[2] = M30;
	M[3] = M40;
	MS[2] = M30A;

	// Do we have a 101 key keyboard.

	KB_101 = ( peekb( 0x0040, 0x0097 ) & 0x10 ) ? TRUE : FALSE;

	//Re-direct stdout so we don't get the ctl-c (^C) crap displayed
	//when an irate user tries it.

	newhandle = fopen( "nul", "w" );
	shandle = dup( 1 );
	if ( dup2( fileno( newhandle ), 1 ) == -1 )  {
		printf( "Cant re-assign stdout.\n" );
		exit( 1 );
	}

	// Capture original vectors for break handlers, put mine in there

	old_brk = getvect( 0x1B );	// Save old Ctl-C vector
	old_brk2= getvect( 0x23 );	// Save old Ctl-Brk vector
	setvect( 0x1B, c_break );	// Put ours in there.
	setvect( 0x23, c_break2 );	// ditto.
	kbflush();
	if ( MouseInstalled )
		PutMouseAt( 1, 1 );	// Corner the mouse
	DisplayOpenInfo();		// Display author info

	fsfill( 1, 1, 1, VidCols, ' ', LightCyan+BlueBG );
	fsfill( VidRows, 1, 1, VidCols,' ', White+LightGrayBG );
	fswriteC( "PowerTerm v0.04(beta)  -  Copyright (c) 1992-94  Byron D Copeland",
		1, VidCols, LightCyan+BlueBG );
	DisplayMainMenuLine();

	// Zero all indexes

	MenOpIdx ^= MenOpIdx;
	MenPosIdx ^= MenPosIdx;
	MSPosIdx ^= MSPosIdx;
	MSOpIdx ^= MSOpIdx;

	DisplayCmdLine ( 0 );

	// Start with first menu

	( * MenuOps[MenOpIdx] ) ();

	if ( MouseInstalled )
		Mshow();

	// Main Menu Loop Control.  All menus functions are controlled
	// from here.  Process will loop until QUIT = TRUE.

	QUIT = FALSE;

	do {
		MenuLevel = 1;
//		kbflush();
		switch( WaitForEvent() )	{

			case CHAR_EVENT:

				ch = toupper( getch() );
				switch( ch )	{

					case SPECIALKEY:

						if ( MouseInstalled )
							Mhide();
						key = getch();
						switch( key )  {

							case F1:

								GiveUserHelp( "M01.HLP" );
								break;

							case ALTQ:

								QUIT = TRUE;
								break;

							case LEFT:

								if ( MenOpIdx < MainMax )
									CloseWindow();
								if ( MenOpIdx == 0 )
									MenOpIdx = MainMax;
								else
									MenOpIdx -= 1;
								MenPosIdx ^= MenPosIdx;
								MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
								( * MenuOps[MenOpIdx] ) ();
								break;

							case RIGHT:

								if ( MenOpIdx < MainMax )
									CloseWindow();
								if ( MenOpIdx == MainMax )
									MenOpIdx ^= MenOpIdx;
								else
									MenOpIdx += 1;
								MenPosIdx ^= MenPosIdx;
								MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
								( * MenuOps[MenOpIdx] ) ();
								break;

							case DN:

								MoveMenuBarDown( M[MenOpIdx], &MenPosIdx,
									&MenPos[MenOpIdx].Row,
									&MenPos[MenOpIdx].RowMin, &MenPos[MenOpIdx].Col,
									&MenPos[MenOpIdx].MaxIdx, LightCyan+CyanBG,
										White+BlackBG ); /* Down Arrow    */
								break;

							case UP:

								MoveMenuBarUp( M[MenOpIdx], &MenPosIdx,
									&MenPos[MenOpIdx].Row,
									&MenPos[MenOpIdx].RowMax, &MenPos[MenOpIdx].Col,
									&MenPos[MenOpIdx].MaxIdx, LightCyan+CyanBG,
										White+BlackBG );   /*  Up Arrow     */
								break;

							case STAB:	// Shft-TAB

								if ( MenOpIdx < MainMax )
									CloseWindow();
								if ( MenOpIdx == 0 )
									MenOpIdx = MainMax;
								else
									MenOpIdx -= 1;
								MenPosIdx ^= MenPosIdx;
								MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
								( * MenuOps[MenOpIdx] ) ();
								break;

							default:
								break;
						}
						if ( MouseInstalled )
							Mshow();
						break;

					case TAB:

						if ( MouseInstalled )
							Mhide();
						if ( MenOpIdx < MainMax )
							CloseWindow ();
						if ( MenOpIdx == MainMax )
							MenOpIdx = 0;
						else
							MenOpIdx += 1;
						MenPosIdx ^= MenPosIdx;
						MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
						( * MenuOps[MenOpIdx] ) ();
						if ( MouseInstalled )
							Mshow();
						break;

					case CR:

						if ( MouseInstalled )
							Mhide();

						if ( ( ( MenOpIdx == 4 ) && ( MenPosIdx == 0 ) ) ||
							( ( MenOpIdx == 3 ) && (MenPosIdx == 1 ) ) )	{
							( * Process[MenOpIdx][MenPosIdx] ) ( &QUIT );
						}
						else	{
							fsattr( MenPos[MenOpIdx].Row,
								MenPos[MenOpIdx].Col, 1, ( strlen( (*M[MenOpIdx]) ) ),
								Gray+LightGrayBG );
							( * Process[MenOpIdx][MenPosIdx] ) ();
							fsattr( MenPos[MenOpIdx].Row,
								MenPos[MenOpIdx].Col, 1, ( strlen( (*M[MenOpIdx]) ) ),
								White+BlackBG );
						}
						DisplayCmdLine( 0 );
						if ( MouseInstalled )
							Mshow();
						break;

					case 'Q':

						if ( MenOpIdx != 4 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 4;
							MenPosIdx ^= MenPosIdx;
							( * MenuOps[MenOpIdx] ) ();
						}
						break;

					case 'I':

						if ( MenOpIdx != 0 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx ^= MenOpIdx;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
						break;

					case 'T':

						if ( MenOpIdx != 1 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 1;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
						break;

					case 'E':

						if ( MenOpIdx != 2 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 2;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
						break;

					case 'S':

						if ( MenOpIdx != 3 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 3;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
						break;

					default:

						Beep( 600 );
						break;
				}
				break;

			case MOUSE1BUTTON:

				if ( MouseInstalled )
					Mhide();
				if ( MouseRow == 2 )	{
					if (( MouseCol >= 2 ) && ( MouseCol <= 5 ))	{
						if ( MenOpIdx != 0 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx ^= MenOpIdx;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
					else
					if (( MouseCol >= 14 ) && ( MouseCol <= 21 ))	{
						if ( MenOpIdx != 1 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 1;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
					else
					if (( MouseCol >= 28 ) && ( MouseCol <= 32 ))	{
						if ( MenOpIdx != 2 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 2;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
					else
					if (( MouseCol >= 40 ) && ( MouseCol <= 45 ))	{
						if ( MenOpIdx != 3 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 3;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
					else
					if (( MouseCol >= 53 ) && ( MouseCol <= 56 ))	{
						if ( MenOpIdx != 4 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 4;
							MenPosIdx ^= MenPosIdx;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
				}
				else
				if ((( MouseRow >= MenPos[MenOpIdx].RowMin ) &&
					( MouseRow <= MenPos[MenOpIdx].RowMax )) &&
					(( MouseCol >= MenPos[MenOpIdx].Col ) &&
					( MouseCol <= MenPos[MenOpIdx].MaxCol )))	{
					MoveMenuBar( M[MenOpIdx], &MenPosIdx,
						&MenPos[MenOpIdx].Row,
						MouseRow,
						&MenPos[MenOpIdx].RowMin, &MenPos[MenOpIdx].Col,
						&MenPos[MenOpIdx].MaxIdx, LightCyan+CyanBG,
							White+BlackBG );
					DisplayCmdLine( 0 );
				}
				else
					Beep( 100 );
				if ( MouseInstalled )
					Mshow();
				break;

			case DOUBLE1CLICK:

				if ( MouseInstalled );
					Mhide();

				if ( MouseRow == 2 )	{
					if (( MouseCol >= 2 ) && ( MouseCol <= 5 ))	{
						if ( MenOpIdx != 0 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx ^= MenOpIdx;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
					else
					if (( MouseCol >= 14 ) && ( MouseCol <= 21 ))	{
						if ( MenOpIdx != 1 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 1;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
					else
					if (( MouseCol >= 28 ) && ( MouseCol <= 32 ))	{
						if ( MenOpIdx != 2 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 2;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
					else
					if (( MouseCol >= 40 ) && ( MouseCol <= 45 ))	{
						if ( MenOpIdx != 3 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 3;
							MenPosIdx ^= MenPosIdx;
							MenPos[MenOpIdx].Row = MenPos[MenOpIdx].RowMin;
							( * MenuOps[MenOpIdx] ) ();
						}
					}
					else
					if (( MouseCol >= 53 ) && ( MouseCol <= 56 ))	{
						if ( MenOpIdx != 4 )  {
							if ( MenOpIdx < MainMax )
								CloseWindow();
							MenOpIdx = 4;
							MenPosIdx ^= MenPosIdx;
							( * MenuOps[MenOpIdx] ) ();
						}
						( * Process[MenOpIdx][MenPosIdx] ) ( &QUIT );
					}
				}
				else
				if ((( MouseRow >= MenPos[MenOpIdx].RowMin ) &&
					( MouseRow <= MenPos[MenOpIdx].RowMax )) &&
					(( MouseCol >= MenPos[MenOpIdx].Col ) &&
					( MouseCol <= MenPos[MenOpIdx].MaxCol )))	{
					MRowSave = MouseRow;

					// Move to menu object pointed to &
					// change color to gray+lightgraybg.

					MoveMenuBar( M[MenOpIdx], &MenPosIdx,
						&MenPos[MenOpIdx].Row,
						MouseRow,
						&MenPos[MenOpIdx].RowMin, &MenPos[MenOpIdx].Col,
						&MenPos[MenOpIdx].MaxIdx, LightCyan+CyanBG,
							Gray+LightGrayBG );
					if (( MenOpIdx == 2 ) && ( MenPosIdx == 2 ))
						( * Process[MenOpIdx][MenPosIdx] ) ( TRUE );
					else
						( * Process[MenOpIdx][MenPosIdx] ) ();

					// Return menu object to normal color.

					fsattr( MRowSave, MenPos[MenOpIdx].Col,
						1, ( strlen( ( * M[MenOpIdx] ) ) ),
						White+BlackBG );
					DisplayCmdLine( 0 );
				}
				else	{
					Beep( 100 );
				}
				if ( MouseInstalled );
					Mshow();
				break;

			default:

				break;
		}
	} 	while ( !QUIT );	      	// End of Main Menu Loop

	// If we have a mouse, turn it off.

	if ( MouseInstalled )	{
		Mhide();
		TurnOffMouse();
	}
	RestoreScreen();			// Display original screen
	setvect( 0x1B, old_brk );		// Restore Ctl-C Break
	setvect( 0x23, old_brk2 );		// Restore Ctl-Break
	CloseCommMem();				// Deallocate comm buffer
	xr.h.ah = 0x02;
	xr.h.bh = 0x00;
	xr.h.dh = RowSave;
	xr.h.dl = ColSave;
	int86( VIDEOINT, &xr, &xr );       	// Restore cursor position
	CursorOn();
	fclose( newhandle );			// Restore stdout
	dup2( shandle, 1 );
	printf( "\n                              P o w e r T e r m\n" );
	printf( "                         C o m m u n i c a t i o n s\n" );
	printf( "                              Version 0.04 (beta)\n\n" );
	printf( "                         Serial Number: %s\n\n", Serial_Number );
	printf( "                    Copyright (c) 1992-94 Byron D Copeland\n" );
	printf( "                              All Rights Reserved.\n\n" );
	printf( "\n" );
	printf( "*******************************************************************************\n");
	printf( "* This version of PowerTerm is restricted to Beta Testers only and is not to  *\n");
	printf( "* be distributed to any other party or parties without the expressed consent  *\n" );
	printf( "* from the author.                                                            *\n" );
	printf( "*******************************************************************************\n");
	return( 0 );
}


/*------------------------------------------------------------------------
	Function to process submenus
-------------------------------------------------------------------------*/

void ProcessSubMenuLvl_1( void )
{
	int		key;
	int		ch;
	int		QUIT;
	int		MRowSave;
	typedef void 	( * MSPROCS ) ();

	MSPROCS	MSProcess[5][7] =	{
		&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,
		&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,
		&SetModemHardware,&SetModemHStrings,&SetModemHMessages,
		&NoOp,&NoOp,&NoOp,&NoOp,
		&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,
		&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp,&NoOp
	};
	QUIT = FALSE;
	DisplayCmdLine( 7 );
	if ( MouseInstalled )
		Mshow();
	do	{
//		kbflush();
		switch( WaitForEvent() )	{

			case CHAR_EVENT:

				ch = toupper( getch() );
				switch( ch )	{

					case ESCAPE:		// User Abort

						QUIT = TRUE;
						break;

					case SPECIALKEY:

						if ( MouseInstalled )
							Mhide();
						key = getch ();
						switch( key )  {

							case DN:

								MoveMenuBarDown (MS[MSOpIdx], &MSPosIdx,
									&MSPos[MSOpIdx].Row,
									&MSPos[MSOpIdx].RowMin, &MSPos[MSOpIdx].Col,
									&MSPos[MSOpIdx].MaxIdx, LightCyan+CyanBG,
									White+BlackBG); /* Down Arrow    */
								break;

							case UP:

								MoveMenuBarUp (MS[MSOpIdx], &MSPosIdx,
									&MSPos[MSOpIdx].Row,
									&MSPos[MSOpIdx].RowMax,
									&MSPos[MSOpIdx].Col,
									&MSPos[MSOpIdx].MaxIdx, LightCyan+CyanBG,
									White+BlackBG);   /*  Up Arrow     */
								break;

							default:

								break;
						}
						if ( MouseInstalled )
							Mshow();
						break;

					case CR:        /* ENTER Key     */

						if ( MouseInstalled )
							Mhide();
						fsattr( MSPos[MSOpIdx].Row,
							MSPos[MSOpIdx].Col, 1, ( strlen( ( * MS[MSOpIdx] ) ) ),
							Gray+LightGrayBG );
						( * MSProcess[MSOpIdx][MSPosIdx] ) ();
						fsattr( MSPos[MSOpIdx].Row,
							MSPos[MSOpIdx].Col, 1, ( strlen( ( * MS[MSOpIdx] ) ) ),
							White+BlackBG );
						DisplayCmdLine( 7 );
						if ( MouseInstalled )
							Mshow();
						break;

					default:

						Beep( 600 );
						break;
				}
				break;

			case MOUSE1BUTTON:

				if ( MouseInstalled )
					Mhide();
				if ( ( ( MouseRow >= MSPos[MSOpIdx].RowMin ) &&
					( MouseRow <= MSPos[MSOpIdx].RowMax ) ) &&
					( ( MouseCol >= MSPos[MSOpIdx].Col ) &&
					( MouseCol <= MSPos[MSOpIdx].MaxCol ) ) )	{
					MRowSave = MouseRow;
					MoveMenuBar( MS[MSOpIdx], &MSPosIdx,
						&MSPos[MSOpIdx].Row,
						MouseRow,
						&MSPos[MSOpIdx].RowMin, &MSPos[MSOpIdx].Col,
						&MSPos[MSOpIdx].MaxIdx, LightCyan+CyanBG,
							White+BlackBG );
				}
				if ( MouseInstalled )
					Mshow();
				break;

			case DOUBLE1CLICK:

				if ( MouseInstalled )
					Mhide();
				if ( ( ( MouseRow >= MSPos[MSOpIdx].RowMin ) &&
					( MouseRow <= MSPos[MSOpIdx].RowMax ) ) &&
					( ( MouseCol >= MSPos[MSOpIdx].Col ) &&
					( MouseCol <= MSPos[MSOpIdx].MaxCol ) ) )	{
					MRowSave = MouseRow;

					//  Move menu bar to pointed to
					//  object & make it gray+lightgraybg.

					MoveMenuBar( MS[MSOpIdx], &MSPosIdx,
						&MSPos[MSOpIdx].Row,
						MouseRow,
						&MSPos[MSOpIdx].RowMin, &MSPos[MSOpIdx].Col,
						&MSPos[MSOpIdx].MaxIdx, LightCyan+CyanBG,
							Gray+LightGrayBG );
					( * MSProcess[MSOpIdx][MSPosIdx] ) ();

					// return menu object to normal color

					fsattr( MRowSave, MSPos[MSOpIdx].Col,
						1, ( strlen( ( * MS[MSOpIdx] ) ) ),
						White+BlackBG );

				}
				DisplayCmdLine( 7 );
				if ( MouseInstalled )
					Mshow();
				break;

			case MOUSE2BUTTON:		// User Abort

				QUIT = TRUE;
				break;

			default:
				break;
		}
	} while ( !QUIT );
	if ( MouseInstalled )
		Mhide();
}


/*---------------------------------------------------------------------
	Function to display Pull-Down Menu 1
----------------------------------------------------------------------*/

void DisplayM10( void )
{
/*  fsattr (2,1,1,VidCols,Blue+LightGrayBG); */
  DisplayMainMenuLine();
  fsattr( 2, 1, 1, 6, White+BlackBG );
  MakeWindow( 3, 1, 4, 20, LightCyan+CyanBG, LightCyan+CyanBG,
	     Shadow, Frame1 );
  DisplayPullDownData( M[MenOpIdx], &MenPos[MenOpIdx].MaxRows,
		      &MenPos[MenOpIdx].RowMin, &MenPos[MenOpIdx].Col,
		      &MenPos[MenOpIdx].Row, &MenPosIdx );
}

/*---------------------------------------------------------------------
	Function to display Pull-Down Menu 2
----------------------------------------------------------------------*/

void DisplayM20( void )
{
/*  fsattr (2,1,1,VidCols,Blue+LightGrayBG); */
  DisplayMainMenuLine();
  fsattr( 2, 13, 1, 10, White+BlackBG );
  MakeWindow( 3, 13, 4, 17, LightCyan+CyanBG, LightCyan+CyanBG,
	     Shadow, Frame1 );
  DisplayPullDownData( M[MenOpIdx], &MenPos[MenOpIdx].MaxRows,
		      &MenPos[MenOpIdx].RowMin, &MenPos[MenOpIdx].Col,
		      &MenPos[MenOpIdx].Row, &MenPosIdx );
}

/*---------------------------------------------------------------------
	Function to display Pull-Down Menu 3
----------------------------------------------------------------------*/

void DisplayM30( void )
{
/*  fsattr (2,1,1,VidCols,Blue+LightGrayBG); */
  DisplayMainMenuLine();
  fsattr( 2, 27, 1, 7, White+BlackBG );
  MakeWindow( 3, 27, 10, 24, LightCyan+CyanBG, LightCyan+CyanBG,
	     Shadow, Frame1 );
  DisplayPullDownData( M[MenOpIdx], &MenPos[MenOpIdx].MaxRows,
		      &MenPos[MenOpIdx].RowMin, &MenPos[MenOpIdx].Col,
		      &MenPos[MenOpIdx].Row, &MenPosIdx );
}


/*---------------------------------------------------------------------
	Function to display Pull-Down Menu 4
----------------------------------------------------------------------*/

void DisplayM40( void )
{
/*  fsattr (2,1,1,VidCols,Blue+LightGrayBG); */
  DisplayMainMenuLine();
  fsattr( 2, 39, 1, 8, White+BlackBG );
  MakeWindow( 3, 39, 4, 19, LightCyan+CyanBG, LightCyan+CyanBG,
	     Shadow, Frame1 );
  DisplayPullDownData( M[MenOpIdx], &MenPos[MenOpIdx].MaxRows,
		      &MenPos[MenOpIdx].RowMin, &MenPos[MenOpIdx].Col,
		      &MenPos[MenOpIdx].Row, &MenPosIdx );
}

/*---------------------------------------------------------------------
	Function to display Pull-Down Sub-Menu 3a
----------------------------------------------------------------------*/

void DisplayM30A( void )
{
	MSPosIdx = 0;
	MSOpIdx = MenOpIdx;
	MSPos[MSOpIdx].Row = MSPos[MSOpIdx].RowMin;
	MakeWindow( 3, 51, 5, 23, LightCyan+CyanBG, LightCyan+CyanBG,
		Shadow, Frame1 );
	DisplayPullDownData( MS[MSOpIdx], &MSPos[MSOpIdx].MaxRows,
			    &MSPos[MSOpIdx].RowMin, &MSPos[MSOpIdx].Col,
			    &MSPos[MSOpIdx].Row, &MSPosIdx );
	ProcessSubMenuLvl_1();		// Process submenu functions
	CloseWindow();
}



/*------------------------------------------------------------------------
	Function to highlight quit menu
-------------------------------------------------------------------------*/

void LightUpQuit( void )
{
/*  fsattr (2,1,1,80,Blue+LightGrayBG); */
  DisplayMainMenuLine();
  fsattr( 2, 52, 1, 6, White+BlackBG );
}

/*------------------------------------------------------------------------
	Function to set QUIT = TRUE.  Result will be
	Termination of program after return.
-------------------------------------------------------------------------*/

void SetQuitOn( int *QuitFlag )
{
  ( *QuitFlag ) = TRUE;
}



// Our ctl-c ctl-brk handler.  Just beep at user, nothing more to do.

void interrupt far c_break( void )
{
	Bog();
}

void interrupt far c_break2( void )
{
	Whoop();
}

// 		Displays the Main Menu Bar

void DisplayMainMenuLine( void )
{
	fsfill( 2, 1, 1, VidCols,' ', Blue+LightGrayBG );
	fswrite( " I", 2, 1, White+LightGrayBG );
	fswriteM( "nfo        ", Blue+LightGrayBG );
	fswriteM( "T", White+LightGrayBG );
	fswriteM( "erminal      S", Blue+LightGrayBG );
	fswriteM( "e", White+LightGrayBG );
	fswriteM( "tup       ", Blue+LightGrayBG );
	fswriteM( "S", White+LightGrayBG );
	fswriteM( "ystem       ", Blue+LightGrayBG );
	fswriteM( "Q", White+LightGrayBG );
	fswriteM( "uit ", Blue+LightGrayBG );
}

//	Function to wait for specific message from modem.

int	WaitForModemMsg( char *str, int secs )
{
	char	msg[50] = "\0";
	char	*mmsg;
	char	*p;
	int		i;
	int		sl;

	TIME_OUT = FALSE;
	KEY_HIT = FALSE;

	sl = strlen( str );
	if ( sl == 0 )
		return( FALSE );
	mmsg = msg;
	i = 0;
	if ( setjmp( ptenv ) )
		return( FALSE );
	if ( signal( SIGTERM, ptsigjmp ) == SIG_ERR )	{
		fprintf( stderr, "WaitForModemMsg() - signal error\n" );
		return( FALSE );
	}
	alarm( secs );						// Set timer
	while ( i++ < 50 )	{
		*mmsg = char_waiting();
		if ( ( p = strstr( msg, str ) ) != NULL )	{
			alarm( 0 );
			*mmsg = '\0';
			return( TRUE );
		}
		mmsg++;
	}
	alarm( 0 );						// Reset timer
	return( FALSE );
}


static void ptsigjmp( int sig )			/* Timer interrupt handler */
{
	if ( sig == SIGTERM )	{
		alarm( 0 );
		if ( signal( SIGTERM, ptsigjmp ) == SIG_ERR )
			fprintf( stderr, "ptsigjmp() - signal error\n" );
		longjmp( ptenv, TRUE );
	}
}


void dosShell( void )
{
	char far 	*savewin;
	char 		*oldprompt, prompt[255];
	char		*curdir;
	char		*p;

	if ( ( savewin = ( char * ) farmalloc(4000L*sizeof( char ) ) ) == NULL )
		return;
	if ( ( curdir = (char *)malloc(128*sizeof(char))) == NULL)	{
		if (savewin)
			farfree(savewin);
		return;
	}
	p=curdir;
	strcpy(p,"X:");
	*p = 'A' + getdisk();    /* replace X with current drive letter */
	getcurdir(0,(p+3));
	gettext (1, 1, 80, 25, (char far *)savewin);
	fclose( newhandle );			// Restore stdout
	dup2( shandle, 1 );
	CursorOn();
	fsfill( 1, 1, VidRows, VidCols, ' ', LightGray+BlackBG );
	oldprompt = getenv( "PROMPT" );
	strcpy( prompt, "PROMPT="
		"     +-------------------------------------------------------+\r\n"
		"     |  Please type EXIT when ready to return to PowerTerm.  |\r\n"
		"     +-------------------------------------------------------+\r\n\r\n$p$g\0" );
	putenv( prompt );
	if ( spawnlp( P_WAIT, getenv( "COMSPEC" ), NULL ) == -1 )	{
		textcolor( 3 );
		sprintf( prompt, "PROMPT=%s", oldprompt );
		putenv( prompt );
		puttext (1, 1, 80, 25, (char far *)savewin);
		CursorOff();
		newhandle = fopen( "nul", "w" );
		shandle = dup( 1 );
		dup2( fileno( newhandle ), 1 );
		if ( savewin )
			farfree( savewin );
		if (curdir)
			free(curdir);
		return;
	}
	textcolor( 3 );
	puttext (1, 1, 80, 25, (char far *)savewin);
	CursorOff();
	newhandle = fopen( "nul", "w" );
	shandle = dup( 1 );
	dup2( fileno( newhandle ), 1 );
	if ( savewin )
		farfree( savewin );
	setdisk( (int)*p-'A');
	chdir(curdir);
	if(curdir)
		free(curdir);
	sprintf( prompt, "PROMPT=%s", oldprompt );
	putenv( prompt );
	return;
}

// End of PT.C