
/*

		---------------------------------------
			    PowerTerm v1.0
		Copyright (c) 1992-93  Byron D Copeland
			 All Rights Reserved.
		---------------------------------------


	Module to handle Xmodem (CheckSum), Xmodem (CRC) and
	Xmodem 1-K File Send & Receive.

	Written by Byron D Copeland

*/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <dir.h>
#include <time.h>
#include <setjmp.h>
#include <signal.h>
#include <conio.h>
#include <alloc.h>
#include <io.h>
#include "fsvlib.h"
#include "crcccitt.h"

#define FALSE		0
#define TRUE		(!FALSE)
#define ACK			0x06
#define NAK			0x15
#define	SOH			0x01
#define STX			0x02
#define ETX			0x03
#define EOT			0x04
#define CAN			0x18
#define CTLX		24
#define ABORT		11
#define BIBI    	10
#define BLK_OK		20

#define updcrc(crc, ch)   (crctab[((crc >> 8) & 255)] ^ (crc << 8) ^ ch)

char *xferr1="Bad CRC            \0";
char *xferr2="Bad Checksum       \0";
char *xferr3="Block Check        \0";
char *xferr4="Time Out           \0";
char *xferr6="Out of Sync        \0";
char *xferr7="Out of Sync        \0";
char *xferr8="\007SIGNAL Error!  \0";
char *xferr9="Wrong Block        \0";
char *xferr10="Transfer did not complete! \0";
char *xferr11="Transfer Complete!\0";
char *xferr12="\007Port Error!   \0";
char *xferr13="Short Block       \0";
char *xferr14="Allocation Error! \0";
char *xferr15="Waiting for NAK   \0";
char *xferr16="Got NAK           \0";
char *xferr17="Cancel by Remote! \0";
char *xferr18="Sending File Hdr  \0";
char *xferr19="Sending File      \0";
char	XMFileDate[30]="\0";
char *protonames[3]={"Xmodem (Checksum)\0","Xmodem (CRC)\0","Xmodem 1-K\0"};

unsigned char		*XmSendBuf;	// Send Block - 1029 bytes.

unsigned short		xmbuff[1+1+1+1024+2+1];

unsigned int		errcnt;
//unsigned short	crc;

int			tmint;
int			WantCRC;
int			XM1K;
int			XmLastBlk;
time_t			xmtstart;
time_t			xmtnow;
struct date		xmdate;
struct time		xmtime;
unsigned long 		xmblknum;
unsigned long		xmtotbytes;
unsigned char		cblknum;
unsigned char		chksum;
unsigned int		XMLEN;
unsigned int		DATALEN;
long			xmfsize;

extern char		*buff;
extern char		*fileswork1;
//extern FILE		*xferfp;
extern int		TIME_OUT;
extern int		KEY_HIT;

jmp_buf			xmenv;
struct	ftime		xmdtinfo;

void 			XmodemRecv( int mode );
void 			XmodemSend( int mode );
static int 		RecvXmblock( FILE *xferfp );
static void 		SendXmBlock( void );
static int 		WaitforXmHandshake( void );
static void 		BuildXmBlock( FILE *xferfp );
static void 		DisplayXMThruPut( void );
static void 		xmsigjmp( int sig );			/* Timer interrupt handler */

/*
 *	Xmodem Receive-File Control Function
 */

void XmodemRecv( int mode )
{
	unsigned int	numtry;
	unsigned int   	MAXTRY;
	int		done;
	int		xmerror;
	char		drv[MAXDRIVE];
	char		dir[MAXDIR];
	char		fn[MAXFILE];
	char		ext[MAXEXT];
	FILE		*xferfp;

	errcnt = 0;
	numtry = 0;
	MAXTRY = 9;
	tmint = 9;
	done = FALSE;
	xmblknum = 1L;		// Always start with blk 1
	xmtotbytes = 0L;
	xmerror = FALSE;
	WantCRC = FALSE;
	XM1K = FALSE;
	TIME_OUT = FALSE;
	KEY_HIT = FALSE;

	flushinput();		// Be sure to start clean.

	if ( signal( SIGTERM, xmsigjmp ) == SIG_ERR )	{
		fprintf(stderr,"xmsigjmp() - signal error\n");
		return;
	}
	if ( mode == 0 )	{
		XMLEN = 132;
		DATALEN = 128;
		SerialOut( NAK );		// Xmodem-CheckSum.
	}
	else
	if ( mode == 1 )	{
		XMLEN = 133;
		DATALEN = 128;
		WantCRC = TRUE;
		SerialOut( 'C' );			// Xmodem-CRC.
	}
	else
	if ( mode == 2 )	{
		XMLEN = 1029;
		DATALEN = 1024;
		WantCRC = TRUE;
		XM1K = TRUE;				// Xmodem-1K-CRC
		SerialOut( 'C' );
	}
	DisplayXferWndw( "R e c e i v e", 0 );
	fnsplit ( fileswork1, drv, dir, fn, ext );
//	sprintf( buff, "%s%s\0", fn, ext );
	fswrite( fileswork1, 9, 22, Yellow+BlueBG );
	fswrite( protonames[mode], 10, 22, Yellow+BlueBG );
	sprintf( buff, "%u\0", numtry );
	fswrite( buff, 15, 22, Yellow+BlueBG );
	sprintf( buff, "%lu\0", xmtotbytes );
	fswrite( buff, 13, 62, Yellow+BlueBG );
	sprintf( buff, "%u\0", DATALEN );
	fswrite( buff, 14, 22, Yellow+BlueBG );
	fswrite( "N/A", 10, 62, Yellow+BlueBG );
	time(&xmtstart);
	if ( ( xferfp = fopen( fileswork1, "wb" ) ) == NULL )	{
		canit();
		done = TRUE;
		xmerror = TRUE;
	}
	setvbuf( xferfp, NULL, _IOFBF, 1024*8 );

	// Loop until ( done )

	while( !done )	{
		sprintf( buff, "%u\0", numtry );
		fswrite( buff, 15, 22, Yellow+BlueBG );
		sprintf( buff, "%lu\0", xmblknum );
		fswrite( buff, 13, 22, Yellow+BlueBG );
		if ( numtry++ >= MAXTRY )	{	// Signal Abort
			canit();
			done = TRUE;
			break;
		}
		switch( RecvXmblock( xferfp ) )	{

			case BLK_OK:

				SerialOut( ACK );
				numtry = 0;
				DisplayXMThruPut();
				break;

			case ABORT:

				canit();
				done = TRUE;
				xmerror = TRUE;
				break;

			case EOT:

				done = TRUE;
				break;

			case CAN:

				done = TRUE;
				xmerror = TRUE;
				break;

			default:
				flushinput();
				SerialOut( NAK );
				break;
		}

	}
	if ( ( done) && ( !xmerror ) )	{
		fswrite( xferr11, 17, 22, Yellow+BlueBG+Blink );
		SerialOut( ACK );
		Ringing2();
	}
	else	{
		fswrite( xferr10, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
	}
	fflush( xferfp );
	fclose( xferfp );
	delay( 1000 );
	CloseWindow();
	return;
}

/*
 *	Xmodem Receive-File Block Function
*/

int RecvXmblock( FILE *xferfp )
{
	register unsigned short crc;
	register unsigned short	*xmbp;

	int		i;
	int		BLK_LEN;


	if ( WantCRC )
		BLK_LEN = XMLEN - 3;
	else
		BLK_LEN = XMLEN - 4;
	if ( setjmp( xmenv ) )
		return( FALSE );
	alarm( tmint );			// Set timer

	// Get a block

	for ( i = 0, xmbp = xmbuff; i < XMLEN; i++, xmbp++ )	{
		*xmbp = char_waiting();
		if ( KEY_HIT )	{

			switch( getch() )	{

				case CTLX:

					alarm( 0 );
					KEY_HIT = FALSE;
					kbflush();
					return( ABORT );

				default:

					kbflush();
					KEY_HIT = FALSE;
					break;
			}
		}
		else
		if ( i == 0 )	{

			if (( *xmbp == CAN ) || ( *xmbp == EOT ))	{
				alarm ( 0 );
				return( *xmbp );
			}
			else

			// Check Xmodem-1k block length change to 128

			if (( XM1K ) && ( WantCRC ) && ( *xmbp == SOH ))	{
				XMLEN = 133;
				XM1K = FALSE;
				DATALEN = 128;
				sprintf( buff, "%u\0", DATALEN );
				fswrite( buff, 14, 22, Yellow+BlueBG );
			}
		}
	}
	alarm( 0 );
	xmbp = xmbuff;

	if ( XM1K )	{
		if ( *xmbp != STX )	{		// Check for Xmodem-1K
			fswrite( xferr7, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}
	else
	if ( *xmbp != SOH )	{
		fswrite( xferr6, 17, 22, Yellow+BlueBG );
		return( FALSE );
	}

	//	Check rest of block

	if ( *(++xmbp) != (xmblknum % 256) )	{	// Right block ?
		if ( *xmbp == (xmblknum % 256) - 1 )	// Previous one ?
			return( TRUE );
		else	{
			fswrite( xferr9, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}
	++xmbp;
	if ( *(xmbp - 1) ^ ( ~*xmbp & 0x00ff ))	{  	// Integrity ?
		fswrite( xferr3, 17, 22, Yellow+BlueBG );
		return( FALSE );
	}

	//	Calc the CheckSum/CRC for the block

	if ( WantCRC )	{
		for ( i = 0, crc = 0; i < BLK_LEN; i++ )
			crc = updcrc( crc, *(++xmbp) );
		if ( crc )	{
			fswrite( xferr1, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}
	else	{
		for ( i = 0, chksum = 0; i < BLK_LEN; i++ )
			chksum += *(++xmbp);
		if ( chksum ^ *(++xmbp) )	{
			fswrite( xferr2, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}

	//	All is good!

	xmbp = &xmbuff[3];
	for ( i = 0; i < DATALEN; i++, xmbp++ )
		fputc( *xmbp, xferfp );
	++xmblknum;
	return( BLK_OK );
}



/*
 *	Xmodem Send-File Control Function
 */

void XmodemSend( int mode )
{
	unsigned int	numtry;
	unsigned int    	MAXTRY;
	int		done;
	int		hsdone;
	int		xmerror;
	int		xmblksent;
	FILE	*xferfp;

	struct		ffblk	fileinfo;

	errcnt = 0;
	numtry = 0;
	MAXTRY = 9;
	tmint = 9;
	done = FALSE;
	hsdone = FALSE;
	xmerror = FALSE;
	TIME_OUT = FALSE;
	KEY_HIT = FALSE;
	xmblknum = 1L;
	xmfsize = 0L;
	xmtotbytes = 0L;
	XM1K = FALSE;
	XmLastBlk = FALSE;

	if ( signal( SIGTERM, xmsigjmp ) == SIG_ERR )	{
		fprintf(stderr,"xmsigjmp() - signal error\n");
		fclose( xferfp );
		return;
	}
	if ( mode == 0 )	{
		XMLEN = 132;
		DATALEN = 128;
	}
	else
	if ( mode == 1 )	{
		XMLEN = 133;
		DATALEN = 128;
	}
	else
	if ( mode == 2 )	{
		XMLEN = 1029;
		DATALEN = 1024;
		XM1K = TRUE;				// Xmodem-1K-CRC
	}

	DisplayXferWndw( "S e n d", 1 );
	findfirst( fileswork1, &fileinfo, FA_ARCH );
	xmfsize = fileinfo.ff_fsize;
//	fswrite( fileinfo.ff_name, 9, 22, Yellow+BlueBG );
	fswrite( fileswork1, 9, 22, Yellow+BlueBG );
	fswrite( protonames[mode], 10, 22, Yellow+BlueBG );
	XmSendBuf = NULL;
	if (( XmSendBuf = ( char * ) malloc( 1032 * sizeof( char ))) == NULL )	{
		canit();
		done = TRUE;
		xmerror = TRUE;
	}
	if ( ( xferfp = fopen( fileswork1, "rb" ) ) == NULL )	{
		canit();
		done = TRUE;
		hsdone = TRUE;
		xmerror = TRUE;
	}
	sprintf( buff, "%u\0", numtry );
	fswrite( buff, 15, 22, Yellow+BlueBG );
	sprintf( buff, "%lu\0", xmtotbytes );
	fswrite( buff, 13, 62, Yellow+BlueBG );
	sprintf( buff, "%u\0", DATALEN );
	fswrite( buff, 14, 22, Yellow+BlueBG );
	sprintf( buff, "%lu\0", xmfsize );
	fswrite( buff, 10, 62, Yellow+BlueBG );
	getftime( fileno( xferfp ), &xmdtinfo );
	xmtime.ti_sec  = xmdtinfo.ft_tsec << 1;
	xmtime.ti_min  = xmdtinfo.ft_min;
	xmtime.ti_hour = xmdtinfo.ft_hour;
	xmdate.da_day  = xmdtinfo.ft_day;
	xmdate.da_mon  = xmdtinfo.ft_month;
	xmdate.da_year = xmdtinfo.ft_year + 1980;
	sprintf( XMFileDate, "%02d-%02d-%02d", xmdate.da_mon,
					   xmdate.da_day,
					   xmdate.da_year - 1900 );
	fswrite( XMFileDate, 11, 62, Yellow+BlueBG );
	fswrite( xferr15, 17, 22, Yellow+BlueBG );

	// Wait for Start-Up NAK

	while ( !hsdone )		{
		sprintf( buff, "%u \0", numtry );
		fswrite( buff, 15, 22, Yellow+BlueBG );
		if ( numtry++ >= MAXTRY )	{	// Signal Abort
			canit();
			done = TRUE;
			hsdone = TRUE;
			break;
		}
		switch( WaitforXmHandshake() )	{

			case 'C':			// Remote wants CRC

				WantCRC = TRUE;
				hsdone = TRUE;
				flushinput();
				break;

			case NAK:			// Remote wants Checksum

				WantCRC = FALSE;
				hsdone = TRUE;
				flushinput();
				break;

			case CAN:

				hsdone = TRUE;
				done = TRUE;
				xmerror = TRUE;
				break;

			case ABORT:

				hsdone = TRUE;
				done = TRUE;
				canit();
				xmerror = TRUE;
				break;

			default:

				break;
		}
	}
	numtry = 0;
	fswrite( "                            ", 17, 22, Yellow+BlueBG );

	time( &xmtstart );

	// Loop until ( done )

	while( !done )	{

		if ( XmLastBlk )	{

			SerialOut( EOT );		// Tell Receiver thats all

			if ( numtry++ >= MAXTRY )	{
				done = TRUE;
				xmerror = TRUE;
				break;
			}
			switch( WaitforXmHandshake() )	{

				case ACK:

					done = TRUE;
					break;

				case ABORT:

					canit();
					done = TRUE;
					xmerror = TRUE;
					break;

				default:

					break;
			}
		}
		else	{

			BuildXmBlock( xferfp );
			xmblksent = FALSE;
			numtry = 0;

		}

		while( !xmblksent )	{
			sprintf( buff, "%u \0", numtry );
			fswrite( buff, 15, 22, Yellow+BlueBG );
			sprintf( buff, "%lu\0", xmblknum );
			fswrite( buff, 13, 22, Yellow+BlueBG );
			if ( numtry++ >= MAXTRY )	{	// Signal Abort
				canit();
				xmerror = TRUE;
				xmblksent = TRUE;
				done = TRUE;
				break;
			}

			SendXmBlock();

			switch(  WaitforXmHandshake() )	{

				case ACK:

					xmblksent = TRUE;
					xmblknum++;
					numtry = 0;
					DisplayXMThruPut();
					if ( xmfsize > 0L )
						XferUpdateProgressBar( &xmfsize, &xmtotbytes );
					break;

				case  NAK:

					flushinput();
					fswrite( xferr16, 17, 22, Yellow+BlueBG );
					delay(2000);
					break;

				case  CAN:

					xmblksent = TRUE;
					done = TRUE;
					fswrite( xferr17, 17, 22, Yellow+BlueBG+Blink );
					flushinput();
					break;

				case ABORT:

					canit();
					done = TRUE;
					xmerror = TRUE;
					xmblksent = TRUE;
					break;

				default:

					break;
			}
		}
	}
	if ( ( done ) && ( !xmerror ) )	{
			fswrite( xferr11, 17, 22, Yellow+BlueBG+Blink );
			SerialOut( ACK );
			Ringing2();
	}
	else	{
		fswrite( xferr10, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
	}
	fclose( xferfp );
	delay( 1000 );
	if ( XmSendBuf )
		free( XmSendBuf );
	CloseWindow();
	return;
}



int WaitforXmHandshake( void )
{
	int	nakchar;

	if ( setjmp( xmenv ) )
		return( FALSE );
	alarm( tmint );				// Set timer
	nakchar = char_waiting();
	if ( KEY_HIT )	{

		switch( getch() )	{

			case CTLX:

				alarm( 0 );
				KEY_HIT = FALSE;
				kbflush();
				return( ABORT );

			default:

				kbflush();
				KEY_HIT = FALSE;
				break;
		}
	}
	alarm( 0 );
	return( nakchar & 0xff );
}



void BuildXmBlock( FILE *xferfp )
{
	register unsigned short crc;
	register unsigned char	*xmbp;

	int		i;
	int		j;
	int		c;


	xmbp = XmSendBuf;

	if ( XM1K )
		*xmbp = STX;
	else
		*xmbp = SOH;

	*(++xmbp) = ( unsigned char ) xmblknum % 256;
	*(++xmbp) = ~( ( unsigned char ) ( xmblknum % 256 ) );

	for ( i = 0, crc = 0, chksum = 0; i < DATALEN; i++ )	{

		if (( c = fgetc( xferfp )) == EOF )	{

			XmLastBlk = TRUE;

			for ( j = i; j < DATALEN; j++ )	{

				*(++xmbp) = 0x1a;		// CPMEOF
				crc = updcrc( crc, *xmbp );
				chksum += *xmbp;

			}

			break;

		}

		*(++xmbp) = c & 0xff;
		crc = updcrc( crc, *xmbp );
		chksum += *xmbp;

	}

	if ( WantCRC )	{

		crc = updcrc( crc, '\0' );
		crc = updcrc( crc, '\0' );

		*(++xmbp) = ( unsigned char ) ( crc >> 8 );
		*(++xmbp) = ( unsigned char ) crc;
	}
	else	{

		*(++xmbp) = chksum;
	}
}


void SendXmBlock( void )
{
	register unsigned char 	*xmbp;
	int		i;

	for ( i = 0, xmbp = XmSendBuf; i < XMLEN; i++ )
		SerialOut( *xmbp++ );

	return;
}


void DisplayXMThruPut( void )
{
	static double	xmsecs;
	static double	xmrate;

	xmtotbytes += ( unsigned long ) DATALEN;
	sprintf( buff, "%lu\0", xmtotbytes );
	fswrite( buff, 13, 62, Yellow+BlueBG );
	time( &xmtnow );

	xmsecs = difftime( xmtnow, xmtstart );
	if ( xmsecs < 1.0 )
		xmsecs = 1.0;
	xmrate = ( double ) xmtotbytes / xmsecs;
	sprintf( buff, "%u    \0", ( unsigned int ) xmrate );
	fswrite( buff, 14, 62, Yellow+BlueBG );
}

static void xmsigjmp( int sig )			/* Timer interrupt handler */
{
	if ( sig == SIGTERM )	{

		alarm( 0 );
		if ( signal( SIGTERM, xmsigjmp ) == SIG_ERR )
			fprintf(stderr,"xmsigjmp() - signal error\n");

		longjmp( xmenv, TRUE );
	}
}
