
/*
		---------------------------------------
		PowerTerm (c) 1992-94  Byron D Copeland
			All Rights Reserved.
		---------------------------------------


	PTYMODEM.C

	Module to handle (True) Ymodem, Ymodem Batch and
	Ymodem-G File Send & Receive.


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <dos.h>
#include <dir.h>
#include <setjmp.h>
#include <signal.h>
#include <conio.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
#include "fsvlib.h"
#include "bcstrs.h"

//#define DEBUG
#define FALSE		0
#define TRUE		(!FALSE)
#define	SOH			0x01
#define STX			0x02
#define EOT			0x04
#define CAN			0x18
#define ACK			0x06
#define NAK			0x15
#define CTLX		24


#define BLKHDR		1
#define	BLKNUM		1
#define BLKCNUM		1
#define MAXBLKSZ	1024
#define BLKCRC		2

#define RTS			0x02
#define MC_INT		0x08
#define MCR         4

#define YMODEMG		2
#define BIBI		10
#define ABORT		11
#define FILEOK		12
#define BLK_OK		20
#define YMETX		30
#define FILEOF		30
#define GOODBLK		0
#define BADFILE		40
#define SYNCERR         300

extern unsigned short 	crctab[256];

#define updcrc( crc, ch ) ( crctab[( ( crc >> 8 ) & 255)] ^ ( crc << 8 ) ^ ch )

// Externals

extern int		TIME_OUT;
extern int		KEY_HIT;
extern int		portbase;
extern int		COM_WAIT;
extern int		TRANS_CLEAR;

extern char 		*filestoxfer[20];
extern char		*WCNames[1000];
extern unsigned int 	filestodo;
extern unsigned int 	fileslefttodo;
extern unsigned long 	actualbaud;
extern char		*buff;
extern char		*TempBuf;

extern struct	PTS	{

	char	PTPaths[66];

} PTSetup[4];

char 			*ymprotonames[3]={"Ymodem \0","Ymodem (Batch)\0",
					  "Ymodem-G\0"};
static unsigned int     flcnt = 0;
unsigned short		ymbuff[BLKHDR+BLKNUM+BLKCNUM+MAXBLKSZ+BLKCRC+1];

unsigned char		*YmSendBuf;

char		ymPath[MAXPATH];

int		YMtmint;
int		YMdone;
int		YMWantCRC;
int		YM1K;
int		FirstNAK = FALSE;
char		YMFileName[128]="\0";
char		YMFileSize[35]="\0";
char		YMFileDate[35]="\0";

char		*ym_msg1  ={ "Sending File Header        " };
char		*ym_msg2  ={ "Waiting for Startup NAK    " };
char		*ym_msg3  ={ "Sending Data Block         " };
char		*ym_msg4  ={ "Waiting for File Header    " };
char		*ym_msg5  ={ "Receiving Data Block       " };
char		*ym_msg6  ={ "File Transfer Successful!  " };
char		*ym_msg7  ={ "File Transfer Unsuccessful!" };
char		*ym_msg8  ={ "Resending Last Block       " };
char		*ym_msg9  ={ "NAK Timeout on Startup     " };
char		*ym_msg10 ={ "Out of Sync                " };
char		*ym_msg11 ={ "Block Sequence Error       " };
char		*ym_msg12 ={ "Block Check Error          " };
char		*ym_msg13 ={ "Block CRC Error            " };
char		*ym_msg14 ={ "Block Checksum Error       " };
char		*ym_msg15 ={ "Got NAK - Resending Block  " };
char		*ym_msg16 ={ "Error on Open File         " };
char		*ym_msg17 ={ "Transfer Aborted by User!  " };
char		*ym_msg18 ={ "Transfer Aborted by Remote!" };


unsigned long 	ymblknum;
unsigned long	ymtotbytes;
long		ymfsize;
long		ymfdate;
unsigned char	ymcblknum;
unsigned char	ymchksum;
int		YmLastBlk;
int		YmLastFile;
int		YMODEMGFLG;
int		YMBATCH;
int		ABORTU;
int		ABORTR;

unsigned int	YMLEN;
unsigned int	YMDATALEN;
unsigned int	wccntr;

struct date		ymdate;
struct time		ymtime;

jmp_buf			ymenv;
time_t			ticknow;
time_t			tickstart;
struct	tm		*ymasc_date;
struct	ftime		ymdtinfo;

FILE			*yfStream;

/*------------------------------------------------------------------------*
 *		Various Function Prototypes                               *
 *------------------------------------------------------------------------*/

void 		YmodemRecv( int mode );
void		YmodemSend( int mode );
void 		YmodemGSend( int mode );
void 		DisplayXferWndw( char *s1, int s2 );
void		canit( void );

int 		RecvYMblock();
int 		WaitForYmHandshake( void );
int		WaitForYMFileName();
void 		DisplayYMThruPut( void );
int 		BuildYMFileBlock();
int 		BuildYMDataBlock();
void 		SendYMBlock( void );
void 		GenYMcrc( void );
void 		ClearXferWndw( void );
void		XferUpdateProgressBar( long *fs, unsigned long *bytes );
static void 	ymsigjmp( int sig );			/* Timer interrupt handler */


/*------------------------------------------------------------------------*

				  - Ymodem Receive Control Function -
   Handles (True) Ymodem, Ymodem Batch & Ymodem-G Receive Protocols.

 *------------------------------------------------------------------------*/

void YmodemRecv( int mode )
{
	unsigned int	numtry;
	unsigned int   	MAXTRY;

	char			*fstr;

	int		filedone;
	int		ymdone;
	int		ymgetfile;
	int		ymerror;

	wccntr = 0;			// Wild card cntr
	flcnt = 0;
	MAXTRY = 9;			// Max number of reties
	YMtmint = 10;			// Timeout wait
	FirstNAK = FALSE;
	ABORTU = ABORTR = FALSE;
	TIME_OUT = FALSE;
	KEY_HIT = FALSE;
	filedone = FALSE;
	ymdone = FALSE;
	ymgetfile = FALSE;
	ymerror = FALSE;
	YMWantCRC = TRUE;
	yfStream = NULL;
	ymfdate = 0L;

	flushinput();		// Be sure to start clean.
	kbflush();
	DisplayXferWndw( "R e c e i v e", 0 );
	fswrite( ymprotonames[mode], 10, 22, Yellow+BlueBG );

	if ( signal( SIGTERM, ymsigjmp ) == SIG_ERR )	{
		CloseWindow();
		fprintf(stderr,"ymsigjmp() - signal error\n");
		return;
	}

	YmSendBuf = NULL;
	if (( YmSendBuf = ( char * )malloc( 1040*sizeof( char ))) == NULL )	{
		canit();
		ymdone = TRUE;
		ymerror = TRUE;
	}

	YMBATCH = ( mode == 1 ) ? TRUE : FALSE;

	numtry = 0;
	ymblknum = 0L;
	sprintf( buff, "%u \0", numtry );
	fswrite( buff, 15, 22, Yellow+BlueBG );
	sprintf( buff, "%lu\0", ymblknum );
	fswrite( buff, 13, 22, Yellow+BlueBG );

	// Loop until no more files

	while( !ymdone )	{
		ymtotbytes = 0L;
		YMLEN = 133;
		YMDATALEN = 128;
		YM1K = FALSE;
		ymblknum = 0L;
		sprintf( buff, "%u \0", YMDATALEN );
		fswrite( buff, 14, 22, Yellow+BlueBG );
		flushinput();
		if ( mode == YMODEMG )	{
			YMODEMGFLG = TRUE;
			SerialOut( 'G' );	// We want Ymodem-G (CRC)
		}
		else	{
			SerialOut( 'C' );	// We want Ymodem-CRC
			YMODEMGFLG = FALSE;
		}
		numtry = 0;
		ymgetfile = FALSE;
		filedone = FALSE;

		do	{

			if ( numtry++ >= MAXTRY )	{ // Signal Abort
				canit();
				ymgetfile = TRUE;
				ymdone = TRUE;
				filedone = TRUE;
				ymerror = TRUE;
				break;
			}

			fswrite( ym_msg4, 17, 22, Yellow+BlueBG );

			switch ( WaitForYMFileName() )	{

				case FILEOK:

					ClearXferWndw();
					numtry = 0;
					fstr = stupr( ymPath );
					fswrite( fstr, 9, 22, Yellow+BlueBG );
					if ( *YMFileSize )
						fswrite( YMFileSize, 10, 62, Yellow+BlueBG );
					if ( *YMFileDate )
						fswrite( YMFileDate, 11, 62, Yellow+BlueBG );

					flushinput();

					if ( mode == YMODEMG )	{
						SerialOut( 'G' );	// We want Ymodem-G
					}
					else	{
						SerialOut( ACK );	// ACK Filename
						SerialOut( 'C' );	// Request first block
					}
					ymgetfile = TRUE;
					break;

				case BIBI:

					if ( mode != YMODEMG )
						SerialOut( ACK );
					ymdone = TRUE;
					ymgetfile = TRUE;
					filedone = TRUE;
					break;

				case  CAN:

					ymdone = TRUE;
					ymgetfile = TRUE;
					filedone = TRUE;
					ABORTR = TRUE;
					ymerror = TRUE;
					break;

				case ABORT:

					kbflush();
					canit();
					ymgetfile = TRUE;
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTU = TRUE;
					break;

				case SYNCERR:

					if ( mode != YMODEMG )	{
						delay(5000);
						SerialOut( NAK );
					}
					else	{
						canit();
						ymgetfile = TRUE;
						filedone = TRUE;
						ymdone = TRUE;
						ymerror = TRUE;
					}
					break;

				default:

					SerialOut( NAK );
					break;
			}

		}	while( !ymgetfile );

		FirstNAK = FALSE;
		ymblknum = 1L;
		numtry = 0;
		time( &tickstart );

		// Loop until file received

		while ( !filedone )	{
			sprintf( buff, "%u \0", numtry );
			fswrite( buff, 15, 22, Yellow+BlueBG);
			sprintf( buff, "%lu     \0", ymblknum );
			fswrite( buff, 13, 22, Yellow+BlueBG );
			if ( numtry++ >= MAXTRY )		{ // Signal Abort
				canit();
				filedone = TRUE;
				ymdone = TRUE;
				ymerror = TRUE;
				break;
			}

			switch ( RecvYMblock() )	{

				case BLK_OK:

					numtry = 0;
					DisplayYMThruPut();
					if ( ymfsize > 0L )
						XferUpdateProgressBar( &ymfsize, &ymtotbytes );
					if ( mode != YMODEMG )
						SerialOut( ACK );
					break;

				case  ABORT:		// Signal Abort

					canit();
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTU = TRUE;
					break;

				case EOT:

					if ( ( !YMBATCH ) && ( mode != YMODEMG ) )	{
						SerialOut( ACK );
						filedone = TRUE;
						ymdone = TRUE;
						numtry = 0;
						break;
					}
					else
					if ( ( YMBATCH ) && ( !FirstNAK ) && ( mode != YMODEMG ) )	{
						SerialOut( NAK );		// for Ymodem
						FirstNAK = TRUE;
					}
					else
					if ( ( YMBATCH ) && ( FirstNAK ) && ( mode != YMODEMG ) )	{
						SerialOut( ACK );
						SerialOut( 'C' );
						filedone = TRUE;
					}
					else
					if ( mode == YMODEMG )	{
						SerialOut( ACK );
						filedone = TRUE;
					}
					numtry = 0;
					break;

				case  CAN:

					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTR = TRUE;
					break;

				case SYNCERR:

					if ( mode == YMODEMG )	{
						canit();
						filedone = TRUE;
						ymdone = TRUE;
						ymerror = TRUE;
					}
					else	{
						delay(5000);
						flushinput();
						SerialOut( NAK );
					}
					break;

				default:


					if ( mode != YMODEMG )	{
						delay(1000);
						flushinput();
						SerialOut( NAK );
					}
					else	{	 // No Ymodem-G Recovery
						canit();
						delay( 500 );
						filedone = TRUE;
						ymdone = TRUE;
						ymerror = TRUE;
					}
					break;

			}	// end switch.

		}	// end while.

		fflush( yfStream );

		// Set file date/time stamp

		if ( ymfdate )	{
			ymdtinfo.ft_tsec = ymtime.ti_sec >> 1;
			ymdtinfo.ft_min = ymtime.ti_min;
			ymdtinfo.ft_hour = ymtime.ti_hour;
			ymdtinfo.ft_day = ymdate.da_day;
			ymdtinfo.ft_month = ymdate.da_mon;
			ymdtinfo.ft_year = ymdate.da_year - 1980;

			setftime( fileno( yfStream ), &ymdtinfo );
		}
		fclose( yfStream );

	} // end while.

	if ( !ymerror )	{
		fswrite( ym_msg6, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
	}
	else
	if ( ABORTU )	{
		fswrite( ym_msg17, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 1000 );
		flushinput();
	}
	else
	if ( ABORTR )	{
		fswrite( ym_msg18, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 1000 );
		flushinput();
	}
	else	{
		flushinput();
		fswrite( ym_msg7, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 1000 );
	}
	if ( YmSendBuf )
		free( YmSendBuf );

	CloseWindow();
	kbflush();
	return;
}

/*
 *	Ymodem Receive Data Block Function.
 *	This function will loop until an entire block is received.
 *	Block size is dependent on YMLEN.
*/


int RecvYMblock( void )
{
	unsigned short ymcrc;
	unsigned short	*ymbp;
	unsigned char	*ymfbp;

	unsigned int	i;
	int		c;
	int		BLK_LEN;


	if ( setjmp( ymenv ) )
		return( FALSE );

	fswrite( ym_msg5, 17, 22, Yellow+BlueBG );

	alarm( YMtmint );		// Set timer

	// Get a block

	for ( i = 0, ymbp = ymbuff; i < YMLEN; i++, ymbp++ )	{

		*ymbp = ( char_waiting() & 0x00ff );

		if ( KEY_HIT )	{

			switch ( getch() )	{

				case CTLX:

					alarm( 0 );
					KEY_HIT = FALSE;
					kbflush();
					return( ABORT );

				default:

					KEY_HIT = FALSE;
					kbflush();
					break;

			}
		}
		if ( i == 0 )	{

			if ( ( *ymbp == CAN ) || ( *ymbp == EOT ) )	{
				alarm( 0 );
				return( *ymbp );
			}
			else

				// Switch to small blocks?

			if ( ( *ymbp == SOH ) && ( YM1K ) )	{
				YMLEN = 133;
				YM1K = FALSE;
				YMDATALEN = 128;
				sprintf( buff, "%u    \0", YMDATALEN );
				fswrite( buff, 14, 22, Yellow+BlueBG );
			}
			else

				// Switch to big blocks?

			if ( ( *ymbp == STX ) && ( !YM1K ) )	{
				YMLEN = 1029;
				YM1K = TRUE;
				YMDATALEN = 1024;
				sprintf( buff, "%u    \0", YMDATALEN );
				fswrite( buff, 14, 22, Yellow+BlueBG );
			}
		}
	}
	alarm( 0 );
	ymbp = ymbuff;
	if ( YM1K ) 	{
		if ( ( unsigned char )*ymbp != STX )	{	// Out of sync!
			fswrite( ym_msg10, 17, 22, Yellow+BlueBG );
			return( SYNCERR );
		}
	}
	else
	if ( ( unsigned char )*ymbp != SOH )	{		// Out of sync!
		fswrite( ym_msg10, 17, 22, Yellow+BlueBG );
		return( SYNCERR );
	}

	//	Check rest of block

	if ( *( ++ymbp ) != ( ymblknum % 256 ) )	{	// Right block ?
		if ( *ymbp == ( ymblknum % 256 ) - 1 )	// Previous one ?
			return( BLK_OK );
		else	{
			fswrite( ym_msg11, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}
	++ymbp;
	if ( *( ymbp - 1 ) ^ ( ~*ymbp & 0x00ff ) )	{  	// Integrity ?
		fswrite( ym_msg12, 17, 22, Yellow+BlueBG );
		return( FALSE );
	}

	//	Calc the CheckSum/CRC for the block

	if ( YMWantCRC )	{
		for ( i = 0, ymcrc = 0; i < YMLEN - 3; i++ )
			ymcrc = updcrc( ymcrc, *( ++ymbp ) );

		if ( ymcrc )	{							// zero result?
			fswrite( ym_msg13, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}
	else	{
		for ( i = 0, ymchksum = 0; i < YMLEN - 4; i++ )
			ymchksum += *( ++ymbp );

		if ( ymchksum ^ *( ++ymbp ) )	{			// zero result?
			fswrite( ym_msg14, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}

	//	Good block, write it!

	ymbp = &ymbuff[3];
	ymfbp = YmSendBuf;

	for ( i = 0; i < YMDATALEN; i++, ymbp++, ymfbp++ )

		*ymfbp = ( unsigned char )*ymbp;

//		fputc( *ymbp, yfStream );

	fwrite( YmSendBuf, (size_t)YMDATALEN, (size_t)1, yfStream );

	++ymblknum;

	return( BLK_OK );
}

/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

int  WaitForYMFileName( void )
{
	unsigned short ymcrc;
	unsigned short *ymbp;

	unsigned int	i;
	char		*p;
	char 		*endptr;
	char		hold[256];

	ymblknum = 0L;

	if ( setjmp( ymenv ) )
		return( FALSE );


	alarm( YMtmint );						// Set timer

	// Get a block

	for ( i = 0, ymbp = ymbuff; i < YMLEN; i++, ymbp++ )	{
		*ymbp = ( char_waiting() & 0x00ff );

		if ( KEY_HIT )		{

			switch ( getch() )	{

				case CTLX:

					alarm( 0 );
					KEY_HIT = FALSE;
					kbflush();
					return( ABORT );

				default:

					KEY_HIT = FALSE;
					kbflush();
					break;

			}
		}
		if ( i == 0 )	{

			// Did we get CAN or EOT from sender?

			if (( *ymbp == CAN ) || ( *ymbp == EOT ))	{
					alarm(0);
					return( *ymbp );
			}
		}
	}
	alarm( 0 );
	ymbp = ymbuff;

	if ( YM1K )	{
		if ( *ymbp != STX )	{
			fswrite( ym_msg10, 17, 22, Yellow+BlueBG );
			return( SYNCERR );				// Out of Sync
		}
	}
	else
	if ( *ymbp != SOH )	{
		fswrite( ym_msg10, 17, 22, Yellow+BlueBG );
		return( SYNCERR );
	}

	//	Check rest of block

	if ( *( ++ymbp ) != ( ymblknum % 256 ) )	{	// Right block ?
		if ( *ymbp == ( ymblknum % 256 ) - 1 )	// Previous one ?
			return( TRUE );
		else	{
			fswrite( ym_msg11, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}
	++ymbp;
	if ( *( ymbp - 1 ) ^ ( ~*ymbp & 0x00ff ) )	{  	// Integrity ?
		fswrite( ym_msg12, 17, 22, Yellow+BlueBG );
		return( FALSE );
	}

	//	Calc the CheckSum/CRC for the block

	if ( YMWantCRC )	{
		for ( i = 0, ymcrc = 0; i < YMLEN - 3; i++ )
			ymcrc = updcrc( ymcrc, *( ++ymbp ) );
		if ( ymcrc )	{							// zero result?
			fswrite( ym_msg13, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}
	else	{
		for ( i = 0, ymchksum = 0; i < YMLEN - 4; i++ )
			ymchksum += *( ++ymbp );

		if ( ymchksum ^ *( ++ymbp ) )	{			// zero result?
			fswrite( ym_msg14, 17, 22, Yellow+BlueBG );
			return( FALSE );
		}
	}

	//	All is good!

	ymbp = &ymbuff[3];

	if ( ( !( unsigned char ) * ( ymbp ) ) ||
			( !( unsigned char ) * ( ymbp + 1 ) ) )
		return( BIBI );

	strcpy( YMFileName, "\0" );
	strcpy( YMFileSize, "\0" );
	strcpy( YMFileDate, "\0" );
	ymfsize = 0L;
	ymfdate = 0L;

	setstrng( hold, '\0', 255 );

	for ( i = 0; i < YMDATALEN; i++, ymbp++ ) 	{
		if ( ! ( unsigned char ) *ymbp )
			*ymbp = ' ';
		hold[i] = ( unsigned char ) *ymbp;
	}

	sscanf( hold, "%s %lu %lo", YMFileName, &ymfsize, &ymfdate );

	if ( ymfsize )
		sprintf( YMFileSize, "%lu\0", ymfsize );
	else
		strcpy( YMFileSize, "n/a" );

// insert here

	if ( ymfdate )	{
		unixtodos( ( long ) ymfdate, &ymdate, &ymtime );
		sprintf( YMFileDate, "%02d-%02d-%02d \0", ymdate.da_mon,
			 ymdate.da_day,
			 ymdate.da_year - 1900 );
	}
	else	{
		strcpy( YMFileDate, "n/a" );
	}

	// Append the file name to our download directory

	if ( *PTSetup[0].PTPaths )	{

		if ( PTSetup[0].PTPaths[strlen( PTSetup[0].PTPaths ) - 1] == '\\' )

			sprintf( ymPath, "%s%s", PTSetup[0].PTPaths, YMFileName );

		else

			sprintf( ymPath, "%s\\%s", PTSetup[0].PTPaths, YMFileName );
	}
	else	{
		strcpy( ymPath, YMFileName );
	}
	if ( ( yfStream = fopen( ymPath, "wb" ) ) == NULL )
		return ( CAN );
	setvbuf( yfStream, NULL, _IOFBF, 1024*8 );
	return( FILEOK );
}

/*------------------------------------------------------------------------*
 *  - Ymodem Send Control Function -                           		  *
 *  Handles (True) Ymodem, Ymodem Batch & Ymodem-G Send Protocols         *
 *------------------------------------------------------------------------*/

void YmodemSend( int mode )
{
	unsigned int	numtry;
	unsigned int   	MAXTRY;
	unsigned int	i;

	int		filedone;
	int		ymdone;
	int		ymsndfile;
	int		ymerror;
	int		ymblksent;
	int		hsa;
	int		hsb;

	yfStream = NULL;
	numtry = 0;
	MAXTRY = 9;
	YMtmint = 9;
	wccntr = 0;
	fileslefttodo = 0;
	KEY_HIT = FALSE;
	ABORTU = ABORTR = FALSE;
	TIME_OUT = FALSE;
	filedone = FALSE;
	ymdone = FALSE;
	ymsndfile = FALSE;
	ymerror = FALSE;
	ymblksent = FALSE;
	YMWantCRC = FALSE;
	YmLastFile = FALSE;
	YmLastBlk = FALSE;
	kbflush();
	DisplayXferWndw( "S e n d", 1 );
	fswrite( ymprotonames[mode], 10, 22, Yellow+BlueBG );

	if ( signal( SIGTERM, ymsigjmp ) == SIG_ERR )	{
		CloseWindow();
		fprintf(stderr,"ymsigjmp() - signal error\n");
		return;
	}

	// Allocate send buffer

	YmSendBuf = NULL;
	if (( YmSendBuf = ( unsigned char * )malloc( 1040*sizeof( char ))) == NULL )	{
		canit();
		ymdone = TRUE;
		ymerror = TRUE;
	}

	// Allocate room for wildcard names.

	for ( i = 0; i < 50; i++ )	{
		if (( WCNames[i] = (char *)malloc(16*sizeof(char))) == NULL)	{
			canit();
			ymdone = TRUE;
			ymerror = TRUE;
			break;
		}
	}

	// Loop until no more files to send

	numtry = 0;
	fswrite( ym_msg2, 17, 22, Yellow+BlueBG );

	while( !ymdone )	{

		ymtotbytes = 0L;
		YMLEN = 133;
		YMDATALEN = 128;
		YM1K = FALSE;
		YmLastFile = FALSE;
		ymblknum = 0L;
		sprintf( buff, "%u \0", YMDATALEN );
		fswrite( buff, 14, 22, Yellow+BlueBG );
//		numtry = 0;
		ymsndfile = FALSE;
		filedone = FALSE;

		do	{
			sprintf( buff, "%u \0", numtry );
			fswrite( buff, 15, 22, Yellow+BlueBG );
			sprintf( buff, "%lu         \0", ymblknum );
			fswrite( buff, 13, 22, Yellow+BlueBG );
			if ( numtry++ >= MAXTRY )	{ // Signal Abort
				canit();
				ymsndfile = TRUE;
				ymdone = TRUE;
				filedone = TRUE;
				ymerror = TRUE;
				break;
			}
			hsa = WaitForYmHandshake();
			switch( hsa )	{

				case 'C':   	// Wants Ymodem
				case 'G':	// Wants Ymodem-G

					flushinput();
					YMWantCRC = TRUE;
					YmLastFile = BuildYMFileBlock();
//					while( ( YmLastFile = BuildYMFileBlock() ) == BADFILE );
					filedone = TRUE;
					break;

				case ACK:

					if ( YmLastFile )	{
						ymsndfile = TRUE;
						filedone = TRUE;
						ymdone = TRUE;
					}
					break;

				case ABORT:

					kbflush();
					canit();
					flushinput();
					ymsndfile = TRUE;
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTU = TRUE;
					break;

				case CAN:

					kbflush();
					flushinput();
					ymsndfile = TRUE;
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTR = TRUE;
					break;

				default:

//					flushinput();
//					fswrite( ym_msg16, 17, 22, Yellow+BlueBG );
//					numtry = 0;

					break;
			}

		}	while( !filedone );
		numtry = 0;
		ClearXferWndw();

		// Sending File Header Loop

		while( !ymsndfile )	{
			sprintf( buff, "%u \0", numtry );
			fswrite( buff, 15, 22, Yellow+BlueBG );
			sprintf( buff, "%lu     \0", ymblknum );
			fswrite( buff, 13, 22, Yellow+BlueBG );
			if ( numtry++ >= MAXTRY )	{ // Signal Abort
				canit();
				ymsndfile = TRUE;
				ymdone = TRUE;
				filedone = TRUE;
				ymerror = TRUE;
				break;
			}

			// Send File Header

			fswrite( ymPath, 9, 22, Yellow+BlueBG );
			fswrite( YMFileSize, 10, 62, Yellow+BlueBG );
			fswrite( YMFileDate, 11, 62, Yellow+BlueBG );
			fswrite( ym_msg1, 17, 22, Yellow+BlueBG );

			SendYMBlock();	// File header

			if ( ( mode == YMODEMG ) && ( YmLastFile ) )	{
				ymsndfile = TRUE;
				ymdone = TRUE;
				filedone = TRUE;
				break;
			}

			hsa = WaitForYmHandshake();
			switch( hsa )	{

				case ACK:

					if ( YmLastFile )	{
						ymsndfile = TRUE;
						filedone = TRUE;
						ymdone = TRUE;
						break;
					}
					hsb = WaitForYmHandshake();
					switch( hsb )	{

						case 'C':
						case 'G':

							ymsndfile = TRUE;
							filedone = FALSE;
							YMLEN = 1029;
							YMDATALEN = 1024;
							YM1K = TRUE;
							break;

						default:
							break;
					}
					break;

				case  CAN:

					ymdone = TRUE;
					ymsndfile = TRUE;
					filedone = TRUE;
					ABORTR = TRUE;
					ymerror = TRUE;
					break;

				case NAK:

					fswrite( ym_msg15, 17, 22, Yellow+BlueBG );
					break;

				case 'G':

					YMLEN = 1029;
					YMDATALEN = 1024;
					YM1K = TRUE;
					ymsndfile = TRUE;
					filedone = FALSE;
					break;

				case ABORT:

					kbflush();
					canit();
//					flushinput();
					ymsndfile = TRUE;
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTU = TRUE;
					break;

				default:

					break;
			}
		}
		YmLastBlk = FALSE;
		ymblknum = 1L;
		numtry = 0;
		fsfill( 18, 22, 1, 50, '', LightCyan+BlueBG );
		fswrite( ym_msg3, 17, 22, Yellow+BlueBG );
		time( &tickstart );

		// Loop until file sent

		while ( !filedone )	{
			sprintf( buff, "%u \0", numtry );
			fswrite( buff, 15, 22, Yellow+BlueBG);
			sprintf( buff, "%lu    \0", ymblknum );
			fswrite( buff, 13, 22, Yellow+BlueBG );
			if ( YmLastBlk )	{		// No more blks to send
				SerialOut( EOT );		// Tell Receiver thats all
				if (numtry++ >= MAXTRY)	{
					ymdone = TRUE;
					ymerror = TRUE;
					filedone = TRUE;
					ymblksent = TRUE;
					break;
				}
				hsa = WaitForYmHandshake();
				switch( hsa )	{

					case NAK:

						numtry = 0;
						ymblksent = TRUE;
//						flushinput();
						break;

					case ACK:

						if ( ymblksent )	{
							filedone = TRUE;
							ymblksent = TRUE;
							fclose( yfStream );
							yfStream = NULL;
						}
						break;

					default:

						break;
				}
			}
			else	{

			// Build next block to send

				YmLastBlk = BuildYMDataBlock();
				ymblksent = FALSE;
			}
			sprintf( buff, "%u \0", YMDATALEN );
			fswrite( buff, 14, 22, Yellow+BlueBG );

			// Loop until block sent successfully

			while ( !ymblksent )	{
				if ( numtry++ >= MAXTRY )		{ // Signal Abort
					canit();
					filedone = TRUE;
					ymblksent = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					break;
				}


				// Send Data Block

				fswrite( ym_msg3, 17, 22, Yellow+BlueBG );

				SendYMBlock();	// Data

				// Ymodem & Ymodem-Batch

				if ( mode != YMODEMG )	{
					switch ( WaitForYmHandshake() )	{

						case ACK:

							numtry = 0;
							ymblksent = TRUE;
							DisplayYMThruPut();
							if ( ymfsize > 0L )
								XferUpdateProgressBar( &ymfsize, &ymtotbytes );
							ymblknum++;
							break;

						case  CAN:

							filedone = TRUE;
							ymdone = TRUE;
							ymblksent = TRUE;
							ABORTR = TRUE;
							ymerror = TRUE;
							break;

						case ABORT:

							canit();
							kbflush();
							filedone = TRUE;
							ymblksent = TRUE;
							ymdone = TRUE;
							ymerror = TRUE;
							ABORTU = TRUE;
							break;

						default:

//							flushinput();
							fswrite( ym_msg8, 17, 22, Yellow+BlueBG );
							break;
					}
				}
				else	{
					if ( fast_kbhit() )	{
						switch( getch() )	{
							case CTLX:
								kbflush();
								filedone = TRUE;
								ymdone = TRUE;
								ymblksent = TRUE;
								ymerror = TRUE;
								ABORTU = TRUE;
								break;

							default:
								kbflush();
								break;
						}
					}
					else
					if ( getccb() ==  CAN )	{
						filedone = TRUE;
						ymdone = TRUE;
						ymblksent = TRUE;
						ymerror = TRUE;
						ABORTR = TRUE;
						break;
					}
					numtry = 0;
					DisplayYMThruPut();
					if ( ymfsize > 0L )
						XferUpdateProgressBar( &ymfsize, &ymtotbytes );
					ymblknum++;
					ymblksent = TRUE;
				}
			}
		}
	}
	if ( yfStream )
		fclose( yfStream );

	if ( !ymerror )	{
		fswrite( ym_msg6, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
	}
	else
	if ( ABORTU )	{
		fswrite( ym_msg17, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 3000 );
//		flushinput();
	}
	else
	if ( ABORTR )	{
		fswrite( ym_msg18, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 3000 );
//		flushinput();
	}
	else	{
		fswrite( ym_msg7, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 3000 );
//		flushinput();
	}


	// De-allocate send buffer

	if ( YmSendBuf )
		free( YmSendBuf );

	// De-allocate wildcard name buffers

	for ( i = 0; i < 50; i++ )	{
		if ( WCNames[i] )
			free( WCNames[i] );
	}
	CloseWindow();
	kbflush();
	return;
}

/*------------------------------------------------------------------------*
 *  - Ymodem-G Send Control Function -                           		  *
 *  Handles (True) Ymodem-G Send Protocols         *
 *------------------------------------------------------------------------*/

void YmodemGSend( int mode )
{
	unsigned int	numtry;
	unsigned int   	MAXTRY;
	unsigned int	i;

	int		filedone;
	int		ymdone;
	int		ymsndfile;
	int		ymerror;
	int		ymblksent;
	int		hsa;
	int		hsb;

	yfStream = NULL;
	numtry = 0;
	MAXTRY = 9;
	YMtmint = 9;
	wccntr = 0;
	flcnt = 0;
	fileslefttodo = 0;
	KEY_HIT = FALSE;
	ABORTU = ABORTR = FALSE;
	TIME_OUT = FALSE;
	filedone = FALSE;
	ymdone = FALSE;
	ymsndfile = FALSE;
	ymerror = FALSE;
	ymblksent = FALSE;
	YMWantCRC = FALSE;
	YmLastFile = FALSE;
	YmLastBlk = FALSE;
	kbflush();
	DisplayXferWndw( "S e n d", 1 );
	fswrite( ymprotonames[mode], 10, 22, Yellow+BlueBG );

	if ( signal( SIGTERM, ymsigjmp ) == SIG_ERR )	{
		CloseWindow();
		fprintf(stderr,"ymsigjmp() - signal error\n");
		return;
	}

	// Allocate send buffer

	YmSendBuf = NULL;
	if (( YmSendBuf = ( unsigned char * )malloc( 1040*sizeof( char ))) == NULL )	{
		canit();
		ymdone = TRUE;
		ymerror = TRUE;
	}

	// Allocate room for wildcard names.

/*	for ( i = 0; i < 50; i++ )	{
		if (( WCNames[i] = (char *)malloc(16*sizeof(char))) == NULL)	{
			canit();
			ymdone = TRUE;
			ymerror = TRUE;
			break;
		}
	}*/

	// Loop until no more files to send

	numtry = 0;
	fswrite( ym_msg2, 17, 22, Yellow+BlueBG );

	while( !ymdone )	{

		ymtotbytes = 0L;
		YMLEN = 133;
		YMDATALEN = 128;
		YM1K = FALSE;
		YmLastFile = FALSE;
		ymblknum = 0L;
		sprintf( buff, "%u \0", YMDATALEN );
		fswrite( buff, 14, 22, Yellow+BlueBG );
//		numtry = 0;
		ymsndfile = FALSE;
		filedone = FALSE;

		do	{
			sprintf( buff, "%u \0", numtry );
			fswrite( buff, 15, 22, Yellow+BlueBG );
			sprintf( buff, "%lu         \0", ymblknum );
			fswrite( buff, 13, 22, Yellow+BlueBG );
			if ( numtry++ >= MAXTRY )	{ // Signal Abort
				canit();
				ymsndfile = TRUE;
				ymdone = TRUE;
				filedone = TRUE;
				ymerror = TRUE;
				break;
			}
			hsa = WaitForYmHandshake();
			switch( hsa )	{

				case 'G':	// Wants Ymodem-G

					flushinput();
					YMWantCRC = TRUE;
					YmLastFile = BuildYMFileBlock();
//					while( ( YmLastFile = BuildYMFileBlock() ) == BADFILE );
					filedone = TRUE;
					break;

				case ACK:

					if ( YmLastFile )	{
						ymsndfile = TRUE;
						filedone = TRUE;
						ymdone = TRUE;
					}
					break;

				case ABORT:

					kbflush();
					canit();
					flushinput();
					ymsndfile = TRUE;
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTU = TRUE;
					break;

				case CAN:

					kbflush();
					flushinput();
					ymsndfile = TRUE;
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTR = TRUE;
					break;

				default:

//					flushinput();
//					fswrite( ym_msg16, 17, 22, Yellow+BlueBG );
//					numtry = 0;

					break;
			}

		}	while( !filedone );
		numtry = 0;
		ClearXferWndw();

		// Sending File Header Loop

		while( !ymsndfile )	{
			sprintf( buff, "%u \0", numtry );
			fswrite( buff, 15, 22, Yellow+BlueBG );
			sprintf( buff, "%lu     \0", ymblknum );
			fswrite( buff, 13, 22, Yellow+BlueBG );
			if ( numtry++ >= MAXTRY )	{ // Signal Abort
				canit();
				ymsndfile = TRUE;
				ymdone = TRUE;
				filedone = TRUE;
				ymerror = TRUE;
				break;
			}

			// Send File Header

			fswrite( ymPath, 9, 22, Yellow+BlueBG );
			fswrite( YMFileSize, 10, 62, Yellow+BlueBG );
			fswrite( YMFileDate, 11, 62, Yellow+BlueBG );
			fswrite( ym_msg1, 17, 22, Yellow+BlueBG );

			SendYMBlock();	// File header

			if ( YmLastFile )	{
				ymsndfile = TRUE;
				filedone = TRUE;
				ymdone = TRUE;
				break;
			}

			hsa = WaitForYmHandshake();
			switch( hsa )	{

				case 'G':

/*					if ( YmLastFile )	{
						ymsndfile = TRUE;
						filedone = TRUE;
						ymdone = TRUE;
						break;
					}*/

					ymsndfile = TRUE;
					filedone = FALSE;
					YMLEN = 1029;
					YMDATALEN = 1024;
					YM1K = TRUE;
					break;

				case  CAN:

					ymdone = TRUE;
					ymsndfile = TRUE;
					filedone = TRUE;
					ABORTR = TRUE;
					ymerror = TRUE;
					break;

				case ABORT:

					kbflush();
					canit();
//					flushinput();
					ymsndfile = TRUE;
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTU = TRUE;
					break;

				default:

					kbflush();
					canit();
//					flushinput();
					ymsndfile = TRUE;
					filedone = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					ABORTU = TRUE;
					break;

			}
		}
		YmLastBlk = FALSE;
		ymblknum = 1L;
		numtry = 0;
		fsfill( 18, 22, 1, 50, '', LightCyan+BlueBG );
		fswrite( ym_msg3, 17, 22, Yellow+BlueBG );
		time( &tickstart );

		// Loop until file sent

		while ( !filedone )	{
			sprintf( buff, "%u \0", numtry );
			fswrite( buff, 15, 22, Yellow+BlueBG);
			sprintf( buff, "%lu    \0", ymblknum );
			fswrite( buff, 13, 22, Yellow+BlueBG );
			if ( YmLastBlk )	{		// No more blks to send
				SerialOut( EOT );		// Tell Receiver thats all
				if (numtry++ >= MAXTRY)	{
					ymdone = TRUE;
					ymerror = TRUE;
					filedone = TRUE;
					ymblksent = TRUE;
					break;
				}
				hsa = WaitForYmHandshake();
				switch( hsa )	{

					case NAK:

						numtry = 0;
						ymblksent = TRUE;
//						flushinput();
						break;

					case ACK:

						if ( ymblksent )	{
							filedone = TRUE;
							ymblksent = TRUE;
							fclose( yfStream );
							yfStream = NULL;
						}
						break;

					default:

						break;
				}
			}
			else	{

			// Build next block to send

				YmLastBlk = BuildYMDataBlock();
				ymblksent = FALSE;
			}
			sprintf( buff, "%u \0", YMDATALEN );
			fswrite( buff, 14, 22, Yellow+BlueBG );

			// Loop until block sent successfully

			while ( !ymblksent )	{
				if ( numtry++ >= MAXTRY )		{ // Signal Abort
					canit();
					filedone = TRUE;
					ymblksent = TRUE;
					ymdone = TRUE;
					ymerror = TRUE;
					break;
				}


				// Send Data Block

				fswrite( ym_msg3, 17, 22, Yellow+BlueBG );

				SendYMBlock();	// Data

				if ( fast_kbhit() )	{
					switch( getch() )	{
						case CTLX:
							kbflush();
							filedone = TRUE;
							ymdone = TRUE;
							ymblksent = TRUE;
							ymerror = TRUE;
							ABORTU = TRUE;
							break;

						default:
							kbflush();
							break;
					}
				}
				if ( getccb() ==  CAN )	{
					filedone = TRUE;
					ymdone = TRUE;
					ymblksent = TRUE;
					ymerror = TRUE;
					ABORTR = TRUE;
					break;
				}
				numtry = 0;
				DisplayYMThruPut();
				if ( ymfsize > 0L )
					XferUpdateProgressBar( &ymfsize, &ymtotbytes );
				ymblknum++;
				ymblksent = TRUE;
			}
		}
	}
	if ( yfStream )
		fclose( yfStream );

	if ( !ymerror )	{
		fswrite( ym_msg6, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
	}
	else
	if ( ABORTU )	{
		fswrite( ym_msg17, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 3000 );
//		flushinput();
	}
	else
	if ( ABORTR )	{
		fswrite( ym_msg18, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 3000 );
//		flushinput();
	}
	else	{
		fswrite( ym_msg7, 17, 22, Yellow+BlueBG+Blink );
		Ringing2();
		delay( 3000 );
//		flushinput();
	}


	// De-allocate send buffer

	if ( YmSendBuf )
		free( YmSendBuf );

	// De-allocate wildcard name buffers

	for ( i = 0; i < flcnt; i++ )	{
		if ( WCNames[i] )
			free( WCNames[i] );
	}
	flcnt = 0;
	CloseWindow();
	kbflush();
	return;
}



/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

int BuildYMFileBlock( void )
{
	register unsigned char *ymbp;

	unsigned long	ymfdateout;
	int				ffdone;
	int				i;
	unsigned char	*p;
	static char		drv[MAXDRIVE];
	static char		dir[MAXDIR];
	static char		fn[MAXFILE];
	static char		ext[MAXEXT];

	char			ymDateWork[32];
	struct	ffblk		fileinfo;
	FILE *dbfp;


	setstrng( YmSendBuf, '\0', YMLEN );
	ymbp = YmSendBuf;
	*ymbp = SOH;

	ymblknum = 0L;
	if ( ( fileslefttodo == filestodo ) && ( wccntr == 0 ) )	{
//		ymblknum = 0L;
		*( ++ymbp ) = ( unsigned char ) ( ymblknum % 256 );
		*( ++ymbp ) = ~( ( unsigned char ) ( ymblknum % 256 ) );
		GenYMcrc();
		return( YMETX );	// Signal Last File
	}
	else	{
		*( ++ymbp ) = ( unsigned char ) ( ymblknum % 256 );
		*( ++ymbp ) = ~( ( unsigned char ) ( ymblknum % 256 ) );
	}

	// This section will load up to 50 wildcard names if wccntr is 0.
	// Up to 20 wilcard file names may be processed per upload session.

	if ( wccntr == 0 )	{
//		for ( i = 0; i < 50; i++ )
//			setstrng( WCNames[i], '\0', 13 );
		for ( i = 0; i < flcnt; i++ )	{
			if (WCNames[i])
				free(WCNames[i]);
		}
		flcnt = 0;
		strcpy( ymPath, filestoxfer[fileslefttodo++] );
		fnsplit( ymPath, drv, dir, fn, ext );
		ffdone = findfirst( ymPath, &fileinfo, FA_ARCH );
//		while( ( !ffdone ) && ( wccntr < 50 ) )		{
		while( !ffdone )	{
			if  ( wccntr > 999 )
				break;
			if (( WCNames[wccntr] = (char *)malloc(13*sizeof(char))) == NULL )	{
				break;
			}
			strcpy( WCNames[wccntr], fileinfo.ff_name );
			ffdone = findnext( &fileinfo );
			wccntr++;
			flcnt++;
		}
	}
	if ( wccntr == 0 )					// shouldn't have gotten this far
		return( BADFILE );
	sprintf( ymPath, "%s%s%s\0", drv, dir, WCNames[--wccntr] );

	#ifdef DEBUG

		fswrite( ymPath, 25, 1, White+BlackBG );

	#endif

	if ( access( ymPath, 0 ) )	{
//		fswrite( ym_msg16, 17, 22, Yellow+BlueBG+Blink );
		return( BADFILE );
	}
	findfirst( ymPath, &fileinfo, FA_ARCH );
//	TempBuf = strlwr( WCNames[wccntr] );
//	strcpy( YMFileName, TempBuf );
	if (( yfStream = fopen( ymPath, "rb" )) == NULL )	{
		fswrite( ym_msg16, 17, 22, Yellow+BlueBG+Blink );
		return( BADFILE );
	}
	sprintf( YMFileSize, "%lu\0", fileinfo.ff_fsize );
	ymfsize = fileinfo.ff_fsize;

	// Add Filename, File Size, File date/time to packet

	++ymbp;
	strcpy( YMFileName, fileinfo.ff_name  );
	p = YMFileName;
	while( *p ) 	{
		*ymbp = tolower( *p );
		ymbp++; p++;
	}

	// Insert file size

	*ymbp++ = '\0';
	p = YMFileSize;
	while( *p )	{
		*ymbp = *p;
		ymbp++; p++;
	}
	getftime( fileno( yfStream ), &ymdtinfo );
	ymtime.ti_sec  = ymdtinfo.ft_tsec << 1;
	ymtime.ti_min  = ymdtinfo.ft_min;
	ymtime.ti_hour = ymdtinfo.ft_hour;
	ymdate.da_day  = ymdtinfo.ft_day;
	ymdate.da_mon  = ymdtinfo.ft_month;
	ymdate.da_year = ymdtinfo.ft_year + 1980;
	sprintf( YMFileDate, "%02d-%02d-%02d", ymdate.da_mon,
					   ymdate.da_day,
					   ymdate.da_year - 1900 );
	ymfdateout = ( unsigned long )dostounix( &ymdate, &ymtime );
	sprintf( ymDateWork, "%lo\0", ymfdateout );

	// Insert file date/time unix format

	*ymbp++ = ' ';
	p = ymDateWork;
	while( *p )	{
		*ymbp = *p;
		ymbp++; p++;
	}
	GenYMcrc();

	#ifdef DEBUG		// Put Block in a file for review

		dbfp = fopen( "Ymodem.dbg", "wb" );
		for ( i = 0; i < YMLEN; i++ )	{
			fputc( YmSendBuf[i], dbfp );
		}
		fclose( dbfp );

	#endif

	return( GOODBLK );
}

/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

int BuildYMDataBlock( void )
{
	register unsigned short ymcrc;
	register unsigned char *ymbp;
	FILE *dbfp;

	int		i;
	int		j;
	int		c;
	int		rc;

	rc = GOODBLK;
//	ymbp = YmSendBuf;
	if ( ( long ) ( (long) ymfsize - ( long ) ymtotbytes ) < 512L )	{
		YM1K = FALSE;
		YMDATALEN = 128;
		if ( YMWantCRC )
			YMLEN = 133;
		else
			YMLEN = 132;
	}
	setstrng( YmSendBuf, '\0', YMLEN );
	ymbp = YmSendBuf;
	if ( YM1K )
		*ymbp = STX;
	else
		*ymbp = SOH;
	*( ++ymbp ) = ( unsigned char ) ( ymblknum % 256 );
	*( ++ymbp ) = ~( ( unsigned char ) ( ymblknum % 256 ) );
	ymbp++;
	for ( i = 0, ymcrc = 0, ymchksum = 0; i < YMDATALEN; i++ )	{
		if ( ( c = fgetc( yfStream ) ) == EOF )	{
			rc = FILEOF;		// Signal Last Block
			for ( j = i; j < YMDATALEN; j++ )	{
				*ymbp = 0x1a;		// CPMEOF
//				ymcrc = updcrc( ymcrc, *ymbp );
//				ymchksum += *ymbp++;
				ymbp++;
			}
			break;
		}
		else	{
			*ymbp = ( unsigned char)c;
			ymbp++;
		}
//		ymcrc = updcrc( ymcrc, *ymbp );
//		ymchksum += *ymbp++;
	}
	GenYMcrc();

	dbfp = fopen( "Ymodem.dbg", "wb" );
	for ( i = 0; i < YMLEN; i++ )	{
		fputc( YmSendBuf[i], dbfp );
	}
	fclose( dbfp );

/*	if ( YMWantCRC )	{
		ymcrc = updcrc( ymcrc, '\0' );
		ymcrc = updcrc( ymcrc, '\0' );
		*ymbp = ( unsigned char )( ymcrc >> 8 );
		*(++ymbp) = ( unsigned char ) ymcrc;
	}
	else	{
		*(++ymbp) = (unsigned char)ymchksum;
	}*/
	return( rc );
}

/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

void GenYMcrc( void )
{
	register unsigned short ymcrc;
	register unsigned char 	*ymbp;

	int						i;

	ymbp = &YmSendBuf[3];
	for ( i = 0, ymcrc = 0, ymchksum = 0; i < YMDATALEN; i++ )	{
		ymcrc = updcrc( ymcrc, *ymbp );
		ymchksum += *ymbp++;
	}
	if ( YMWantCRC )	{
		ymcrc = updcrc( ymcrc, '\0' );
		ymcrc = updcrc( ymcrc, '\0' );
		*ymbp = ( unsigned char )( ymcrc >> 8 );
		*(++ymbp) = (unsigned char) ymcrc;
	}
	else	{
		*ymbp = ( unsigned char )ymchksum;
	}
}

/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

void SendYMBlock( void )
{
	register unsigned char *ymbp;

	int		i;
//	while ( !TRANS_CLEAR );
	for ( i = 0, ymbp = YmSendBuf; i < YMLEN; i++ )	{
//		while ( COM_WAIT );
		while ( SerialOut( *ymbp ) == -1 );
		ymbp++;
	}
}

/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

int WaitForYmHandshake( void )
{
	int	nakchar;

	if ( setjmp( ymenv ) )
		return( FALSE );

	alarm( YMtmint );				// Set timer
	nakchar = char_waiting();
	if ( KEY_HIT )	{
		switch ( getch() )	{

			case CTLX:

				alarm( 0 );
				KEY_HIT = FALSE;
				kbflush();
				return( ABORT );

			default:

				KEY_HIT = FALSE;
				kbflush();
				break;

		}
	}
	alarm( 0 );
	return( nakchar );
}


/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

void DisplayXferWndw( char *s1, int s2 )
{
	MakeWindow( 5, 5, 17, 70, LightCyan+BlueBG, LightCyan+BlueBG,
			Shadow, Frame1 );
	fswrite( "", 7, 5, LightCyan+BlueBG );
	fswrite( "", 7, 74, LightCyan+BlueBG );
	fsfill( 7, 6, 1, 68, '', LightCyan+BlueBG );
	fswrite( "", 19, 5, LightCyan+BlueBG );
	fswrite( "", 19, 74, LightCyan+BlueBG );
	fsfill( 19, 6, 1, 68, '', LightCyan+BlueBG );

	sprintf( buff, "%s   F i l e   i n   P r o g r e s s\0", s1 );
	fswriteC( buff,	6, VidCols, LightCyan+BlueBG );
	fswriteC( "Press ctl-x to abort transfer", 20, VidCols,
			LightCyan+BlueBG );
	fswrite( "        File :", 9, 7,LightCyan+BlueBG );

	fswrite( "    Protocol :", 10, 7,LightCyan+BlueBG );
	fswrite( "     File Size :", 10, 45, LightCyan+BlueBG );
	fswrite( "     File Date :", 11, 45, LightCyan+BlueBG );

	fswrite( "Block Number :", 13, 7, LightCyan+BlueBG );
	if ( s2 == 1 )
		fswrite( "    Bytes Sent :", 13, 45, LightCyan+BlueBG );
	else
	fswrite( "Bytes Received :", 13, 45, LightCyan+BlueBG );

	fswrite( "  Block Size :", 14, 7, LightCyan+BlueBG );
	fswrite( "   Average cps :", 14, 45, LightCyan+BlueBG );
	fswrite( "  Efficiency % :", 15, 45, LightCyan+BlueBG );
	if ( !actualbaud )			// prevent divide by zero!
		fswrite( "n/a", 15, 62, Yellow+BlueBG );

	fswrite( "Block Errors :", 15, 7, LightCyan+BlueBG );
	fswrite( "Total Errors :", 16, 7, LightCyan+BlueBG );
	fswrite( "    Last Msg :", 17, 7, LightCyan+BlueBG );
	fswrite( "    Progress :",18,7,LightCyan+BlueBG );
	fsfill( 18, 22, 1, 50, '', LightCyan+BlueBG );

}

/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

void DisplayYMThruPut()
{
	double		ymsecs;
	double		ymrate;
	double		eff;

	ymtotbytes += (unsigned long)YMDATALEN;
	sprintf( buff, "%lu   \0", ymtotbytes);
	fswrite( buff, 13, 62, Yellow+BlueBG );
	time( &ticknow );
	ymsecs = difftime(ticknow, tickstart);
	if ( ymsecs < 1.0 )
		ymsecs = 1.0;
	ymrate = (double)ymtotbytes / ymsecs;
	sprintf( buff, "%u     \0", (unsigned int)ymrate );
	fswrite( buff, 14, 62, Yellow+BlueBG );
/*	if ( actualbaud )	{			// prevent divide by zero!
		eff = ( ymrate / ( double )( ( double )actualbaud / ( double )10.0 ) ) * ( double )100.0;
		sprintf( buff, "%5.1f\0", eff );
		fswrite( buff, 15, 62, Yellow+BlueBG );
	}
	else	*/
	fswrite( "n/a", 15, 62, Yellow+BlueBG );
}


void XferUpdateProgressBar( long *fs, unsigned long *bytes )
{
	unsigned int 	ProgressBarCols;
	float	    	eff;


	if ( ( * fs ) <= 128L )
		ProgressBarCols = 50;
	else
		ProgressBarCols = abs ((( ( float )( *bytes ) / ( float )( *fs ) ) *
			( float )100.0 ) / ( float )2.0 );
	if ( ProgressBarCols > 50 )
		ProgressBarCols = 50;

//	fsfill( 18, 22, 1, ProgressBarCols, '', Yellow+BlueBG );
	fsfill( 18, 22, 1, ProgressBarCols, '', Yellow+BlueBG );
}


/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

void canit()
{
	int	i;

	for ( i = 0; i < 8; i++ )	{
		SerialOut( CAN );
		delay( 50 );
	}
}

/*------------------------------------------------------------------------*
 *		                                                          *
 *------------------------------------------------------------------------*/

void ClearXferWndw( void )
{
	fswrite( "                                           ", 9, 22, Yellow+BlueBG );
	fswrite( "            ", 10, 62, Yellow+BlueBG );
	fswrite( "            ", 13, 22, Yellow+BlueBG );
	fswrite( "            ", 13, 62, Yellow+BlueBG );
	fswrite( "            ", 14, 22, Yellow+BlueBG );
	fswrite( "            ", 14, 62, Yellow+BlueBG );
	fswrite( "            ", 15, 62, Yellow+BlueBG );
	fswrite( "            ", 15, 22, Yellow+BlueBG );
	fswrite( "            ", 16, 22, Yellow+BlueBG );
	fswrite( "                                 ", 17, 22,
		Yellow+BlueBG );
	fsfill( 18, 22, 1, 50, '', LightCyan+BlueBG );
}


static void ymsigjmp( int sig )			/* Timer interrupt handler */
{
	if ( sig == SIGTERM )	{

		alarm( 0 );
		if ( signal( SIGTERM, ymsigjmp ) == SIG_ERR )
			fprintf( stderr, "ymsigjmp() - signal error\n" );

		longjmp( ymenv, TRUE );
	}
}
