
/*
		---------------------------------------
		PowerTerm (c) 1992-94  Byron D Copeland
			  All Rights Reserved.
		---------------------------------------


		Zmodem.h

*/

#define 	FALSE		0
#define 	TRUE		( !FALSE )

#define		BS		  	8
#define		LF		  	10
#define		CR			13
#define		DLE			16
#define		XON			17
#define		XOFF		19
#define		CAN			24
#define		ESC			27

#define		ZMAXHLEN  	16
#define		ZATTNLEN  	32

#define		ZPAD	  	'*'     // ZPAD starts every header
#define		ZDLE	  	24	// The general purpose escape code
#define		ZBIN	  	'A'	// Starts a binary CRC-16 frame
#define		ZHEX	  	'B'	// Starts a HEX CRC-16 frame
#define		ZBIN32	  	'C'	// Starts a binary CRC-32 frame
#define		ZMAXSPLEN 	1024	// Max sub-packet length

// Variuos frame types

#define		ZRQINIT		0	// Request for receiver init frame
#define		ZRINIT		1	// A receiver init frame
#define		ZSINIT		2	// A sender init frame
#define		ZACK		3	// General purpose acknowledge
#define		ZFILE		4	// File name and data
#define		ZSKIP		5	// Skip incoming file
#define		ZNAK		6	// General purpose not acknowledge
#define		ZABORT		7	// Abort session
#define		ZFIN		8	// Session is complete
#define		ZRPOS		9	// Request to start sending from an
					// address
#define		ZDATA		10	// Start of data frame
#define		ZEOF		11	// End of file
#define		ZFERR		12	// Fatal read or write error D
#define		ZCRC		13	// Request for CRC value of a file
#define		ZCHALLENGE	14	// Challenge frame
#define		ZCOMPL		15	// Request is complete
#define		ZCAN		16	// Remote end cancelled
#define		ZFREECNT	17	// Request for free bytes on filesystem
#define		ZCOMMAND	18	// Command from sending program
#define		ZSTDERR		19	// Output to stderr data follows

#define		UCAN		200	// User cancel return code.  Used to
					// abort transfers.

// Escape sequences in binary data

#define		ZCRCE		'h'	// CRC next, frame ends, header follows
#define		ZCRCG		'i'	// CRC next, frame continues nonstop
#define		ZCRCQ		'j'	// CRC next, frame conts., expect ZACK
#define		ZCRCW		'k'	// CRC next, ZACK, expected, end of
					// frame
#define		ZRUB0		'l'	// Translate to rubout, 0x7f
#define		ZRUB1		'm'	// Translate to rubout, 0xff

// zmReadDataFrame() return values

#define		GOTFLAG	 0x100
#define		GOTCRCE	 ( ZCRCE | GOTFLAG ) 	// ZDLE-ZCRCE received
#define		GOTCRCG	 ( ZCRCG | GOTFLAG )	// ZDLE-ZCRCG received
#define		GOTCRCQ	 ( ZCRCQ | GOTFLAG )	// ZDLE-ZCRCQ received
#define		GOTCRCW	 ( ZCRCW | GOTFLAG )	// ZDLE-ZCRCW received
#define		GOTCAN	 ( CAN   | GOTFLAG )	// can*5 received

// Byte position within header array

#define		ZF0		 3	// Position 0 in the flags array
#define		ZF1		 2	// Position 1 in the flags array
#define		ZF2		 1	// Position 2 in the flags array
#define		ZF3		 0	// Position 3 in the flags array
#define		ZP0		 0	// Low order 8 bits of position
#define		ZP1		 1
#define		ZP2		 2
#define		ZP3		 3

// Bit masks for ZRINIT flags byte ZF0

#define		CANFDX		 0x01		// Rx can send and receive true full
						// duplex
#define		CANOVIO		 0x02		// Rx can receive data during disk io
#define		CANBRK		 0x04		// Rx can send a break signal
#define		CANRLE		 0x08		// Rx can decode RLE
#define		CANLZW		 0x10		// Rx can uncompress
#define		CANFC32		 0x20		// Rx can use 32 bit CRC
#define		ESCCTL		 0x40		// Rx expects ctl chars to be
						// escaped
#define		ESC8		 0x80		// Rx expects 8th bit to be
						// escaped

// Miscellaneous #defineants

#define		OK		 0
#define		ERROR		 -1
#define		TIMEOUT		 -2
#define		GARBAGE_COUNT 	 -3

int		receiver_wants_crc32;
int		current_frame_uses_crc32;
int		last_char_sent;
int		file_at_eof;
int		wake_up_sender_header_type;
int		Rxcount;

long		received_file_position;
long		transmitted_file_position;
long		last_sync_position;
long		last_reported_position;
long		file_length;
long		byte_count;
long		zmfdate;

unsigned int	receiver_buffer_length;
unsigned int	zmblksize = 1024;

unsigned short	zmCRC16;

unsigned long	zmCRC32;
unsigned long 	zmtotalbytes;
unsigned long	zmblknum;

char			received_header[ ZMAXHLEN ];
char			transmitted_header[ ZMAXHLEN ];
char			attention_string[ ZATTNLEN + 1 ];

char			zmBuffer[1040];
char			zmPath[MAXPATH];
char			zmFileDate[35] = "\0";

char			*zm_msg1 = { "Skipping File              " };
char			*zm_msg2 = { "Receiving File Header      " };
char			*zm_msg3 = { "Data Packet Timeout        " };
char			*zm_msg4 = { "Receiving File Data        " };
char			*zm_msg5 = { "Sending ZRPOS              " };
char			*zm_msg6 = { "Received ZPROS             " };

static	char	CancelString[] = {
		CAN, CAN, CAN, CAN, CAN, CAN, CAN, CAN, CAN, CAN,
		BS,  BS,  BS,  BS,  BS,  BS,  BS,  BS,  BS,  BS, '\0' };

static char	*Zendnames[] = { "ZCRCE","ZCRCG","ZCRCQ","ZCRCW"};

static char	*zFrameTypes[]= {

	"No Response to Error Correction Request",
	"No Carrier Detect",
	"TIMEOUT",
	"ERROR",
	"ZRQINIT", 	"ZRINIT", 	"ZSINIT", 		"ZACK",
	"ZFILE",	"ZSKIP",   	"ZNAK",   		"ZABORT",
	"ZFIN", 	"ZRPOS",	"ZDATA",   		"ZEOF",
	"ZFERR",	"ZCRC", 	"ZCHALLENGE",		"ZCOMPL",
	"ZCAN",   	"ZFREECNT",	"ZCOMMAND", 		"ZSTDERR" };


FILE	*zmflfp = NULL;

jmp_buf			zmenv;


time_t			zticknow;
time_t			ztickstart;

struct date		zmdate;
struct time		zmtime;
struct ftime		zmfdtinfo;

extern int		KEY_HIT;

extern unsigned long 	speed;
extern unsigned int	filestodo;
extern unsigned int	fileslefttodo;

extern char 		*filestoxfer[20];
extern char		*WCNames[1000];
extern char		*buff;
extern char		*TempBuf;
extern char		*ym_msg3;
extern char		*ym_msg6;
extern char		*ym_msg7;
extern char		*ym_msg17;
extern char		*ym_msg18;

extern struct	PTS	{

	char	PTPaths[66];

} PTSetup[4];


extern unsigned short 	crctab[256];
extern unsigned long	crc_32_tab[256];

#define Crc16( crc, ch ) ( crctab[( ( crc >> 8 ) & 255)] ^ ( crc << 8 ) ^ (ch & 0xff) )
#define Crc32(octet, crc) (crc_32_tab[(((int)crc) ^ (octet)) & 0xff] ^ ((crc) >> 8)&0x00FFFFFFL)

// 	Function prototypes

int    	zmReceive( void );
int    	zmSend( void );
int	zmWakeUpSender( void );
int 	zmReceiveFiles( void );
int 	zmReceiveSingleFile( void );
void 	zmSendHexHeader( int len, int type, char *hdr );
void 	zmSendBinaryHeader( int length, int type, char *header );
int 	zmReadHeader( char * header );
int	zmReadDataFrame( char *buf, int length );
int	zmReadDataFrameCRC32( char *buf, int length );
int	zmReadDataFrameCRC16( char *buf, int length );
void	zmSendAttentionString( void );
void	zmAckZFIN( void );
int	zmOpenInputFile( char *data );
void	zmPackLongIntoHeader( long header_data );
long	zmUnpackHeaderIntoLong( char *header );
void	zmSendHexEncodedChar( int c );
void	zmSendEncodedChar( int c );
void	zmSendChar( int c );
void	zmSendZFIN( void );
void	zmSendBrk( void );
int	zmReadHexHeaderCRC16( char *header );
int	zmReadBinaryHeaderCRC32( char *header );
int	zmReadBinaryHeaderCRC16( char *header );
int	zmReadEncodedByte( void );
int	zmReadUnencodedByte( void );
int	zmReadHexByte( void );
int	zmReadChar( long timeout );
int	zmGetRinitHeader( void );
int	zmSendSingleFile( char *zfname );
int	zmSendFileContents( void );
void 	zmSendDataFrame( char *buffer, int length, int frameend );
int	zmSyncWithReceiver( int flag );
static 	void zmsigjmp( int sig );			/* Timer interrupt handler */
void 	DisplayZMThruPut( long *bytes );







