/*------------------------------------------------------------------------
  Routine to handle terminal mode operations.
-------------------------------------------------------------------------*/

void TerminalOps( void )
{
	int		QUIT;
	int		c;
	int		ch;
	int		key;
	int     CTSOK;
	int		SHUTDOWN_SERIAL;

	char		*pp;
	void		( * VideoHandler ) ( int c );			// Emulation funtion pointer
	void		( * SendFunctionKey ) ( int key );		// Funtion key function pointer
	void		( * SendTab ) ();				// Tab key function pointer

	// Struct for macro keys

	extern struct MACROS  {
		char			*mname;		// 8 Bytes
		char			*mdesc;		// 60 Bytes
		char			*afkey[12];
		char			*cfkey[12];
	} MacroKeys;

	union REGS toxr;

	INSMODE = FALSE;
	SHUTDOWN_SERIAL = TRUE;
	CTSOK = FALSE;
	LFON = FALSE;
	ROLL = TRUE;
	ECHOON = FALSE;
	VIP = FALSE;
	OLDVMODE = 0;
	VIPLOCAL = FALSE;
	ONLINE = FALSE;
	VIPHIDE = FALSE;
	MacChk = FALSE;
	FnChk = FALSE;
	QUIT = FALSE;
	EmuMode = 0;				// Start with a TTY Terminal
	textcolor( 3 );

	// Set up emulation pointers

	VideoHandler = Term[EmuMode].pointer;
	SendFunctionKey = Fkey[EmuMode].pointer;
	SendTab = Tkey[EmuMode].pointer;

	// Cursor at home

	VTr = 1;
	VTc = 1;

	VTwt = 1;				// Window Top
	VTwb = 24;				// Window Bottom

	SaveTermScreen();
	VTa = 0x03;
	fsfill( 1, 1, VidRows, VidCols, ' ', Cyan+BlackBG );
	CursorOn();
	gotoxy( VTc, VTr );

	// Start up with default port settings

	speed = ModemParms.defs[1];
	port = ( unsigned int ) ModemParms.defs[0];
	parity = ParityData2[( ( unsigned int ) ModemParms.defs[3] )];
	bits = ( int )ModemParms.defs[2];
	stopbits = ( int ) ModemParms.defs[4];

	TimeOutV = ( int ) ModemParms.defs[6];

	// Initialize Serial with Default settings.

	FIFO = FALSE;

	if ( SetSerial( port,
		( unsigned long )speed, parity, bits, stopbits ) != 0 )	{
		fprintf( stderr, "\n!!!! Commmunications Port Initialization Error !!!!!\n" );
		Bog();
		fprintf( stderr, "Please ensure that the com port exists.  Hit any key to continue...\n" );

		ch = getch();
		ClearKeyBoard();
		RestTermScreen();
		CursorOff();
		return;
	}
	initserial();

	flushinput();

	DisplayStatusLine();

	fswrite( "PowerTerm v0.01  Copyright (c) 1992-93  Byron D Copeland", VTr++, VTc, VTa);
	if ( FIFO )
		fswrite( "FIFO support enabled!", VTr++, VTc, VTa );
//	else
//		fswrite( "FIFO support disabled!", VTr++, VTc, VTa );
	fsfill( VTr++, 1, 1, VidCols, '', VTa );
	gotoxy( VTc, VTr );

	// If we have something to talk to, then try to send the init
	// strings.

	CTSOK = ( ( inportb( portbase + MSR ) & CTS ) == 0 ) ? FALSE : TRUE;
	if ( !CTSOK )	{
		MakeWindow( 10, 20, 4, 40, LightCyan+BlueBG, LightCyan+BlueBG,
			Shadow, Frame1 );
		fswriteC( "CTS is not active. Waiting...", 11, VidCols, Yellow+BlueBG );
		fswriteC( "Press any key to CANCEL wait.", 12, VidCols, Yellow+BlueBG );
		do	{
			CTSOK = ( ( inportb( portbase + MSR ) & CTS ) == 0 ) ? FALSE : TRUE;
		}	while( ( !CTSOK ) && ( !kbhit() ) );
		CloseWindow();
		ClearKeyBoard();
	}
	if ( CTSOK )	{
		fswrite( "Initializing...", VTr, VTc, VTa );
		gotoxy( 16, VTr );
		delay( 800 );
		setstrng( TempBuf, '\0', 50 );
		sprintf( TempBuf, "%s\0", ModemParms.inits[9] );
		if ( *TempBuf ) 	{

			while( ( pp = strstr( TempBuf, "|" ) ) != NULL )
				 *pp='\r';

			SerialString( TempBuf );

			// Throw out anything echoed!

			flushinput();

			delay( ModemParms.defs[5] * 100 );
			sprintf( TempBuf, "%s\0", ModemParms.modemmsg[0] );
			if ( *TempBuf ) 	{

				while( ( pp = strstr( TempBuf, "|" ) ) != NULL )
					*pp='\r';

				if ( !WaitForModemMsg( TempBuf, 10 ) )
					fswriteM( "x", VTa );
				else
					fswriteM( ".", VTa );
				gotoxy( 17, VTr );
			}
		}
	}
	flushinput();
	CTSOK = ( ( inportb( portbase + MSR ) & CTS ) == 0 ) ? FALSE : TRUE;
	if ( CTSOK )	{
		setstrng( TempBuf, '\0', 50 );
		sprintf( TempBuf, "%s\0", ModemParms.inits[10] );
		if ( *TempBuf ) 	{

			while( ( pp = strstr( TempBuf, "|" ) ) != NULL )
				*pp='\r';

			SerialString( TempBuf );

			// Throw out anything echoed!

			flushinput();

			delay(ModemParms.defs[5] * 100);
			sprintf( TempBuf,"%s\0",ModemParms.modemmsg[0] );
			if ( *TempBuf ) 	{
				while( ( pp = strstr( TempBuf, "|" ) ) != NULL )
					*pp='\r';
				if ( !WaitForModemMsg( TempBuf, 10 ) )
					fswriteM( "x", VTa );
				else
					fswriteM( ".", VTa );
			}
		}
	}



	fswrite( "PowerTerm Ready!", ++VTr, VTc, VTa );
	VTr++;
	gotoxy( VTc, VTr );

	// Send the ATTENTION string.

	sprintf( buff, "%s\0", ModemParms.inits[4] );

	while( ( pp = strstr( buff, "|" ) ) != NULL )
		*pp = '\r';

	CTSOK = ( ( inportb( portbase + MSR ) & CTS ) == 0 ) ? FALSE : TRUE;
	if ( CTSOK )    {
		if ( *buff )
			SerialString( buff );
	}

	//  Terminal Emulator Loop
	//  Loop until OODone = TRUE

	do	{



		// Poll Keyboard Buffer

		if ( KB_101 )
			toxr.h.ah = 0x11;
		else
			toxr.h.ah = 0x01;
		int86( 0x16, &toxr, &toxr );

		if ( !( toxr.x.flags & 0x40 ) )	{	// key hit?

			if ( ( VIP ) && ( KB_101 ) )	{
				if ( toxr.h.ah == 0xe0 )	{
					if ( toxr.h.al == 0x0d ) 	{	// Keypad Enter
						SerialString( "\033i\0" );
						ClearKeyBoard();
						continue;
					}
				}
			}
			ch = getch();
			if ( ch == SPECIALKEY )  {
				switch( key = getch() )  {

				  case ALT_EQ:		// Alt + =

					if ( ( VIP ) && ( !KB_101 ) )
						SerialString( "\033i\0" );
					break;

				  case ALTA:

					if ( VIP )
						AUTOLF = ( AUTOLF ) ? FALSE : TRUE;
					break;

				  case ALTB:			// Macros

					CursorOff();
					EditMacroKeys();
					CursorOn();
					break;

				  case ALTD:			// Dialing Directory

					if ( !ONLINE )  {
						CursorOff();
						ONLINE = DisplayDialDirectory();
//							ONLINE = CARRIERON;
						fsfill( VidRows, 1, 1, VidCols, ' ', LightGray+BlackBG );
						if ( VIP )  {
							VIPLOCAL = FALSE;
							AUTOLF = FALSE;
						}

						VideoHandler = Term[EmuMode].pointer;
						if ( VIP )
							VTa = 0x02;
						else
							VTa = 0x03;
						SendFunctionKey = Fkey[EmuMode].pointer;
						SendTab = Tkey[EmuMode].pointer;
						CursorOn();
					}
					break;

				  case ALTE:			// Emulation

					CursorOff();
					EmuMode = SwitchEmulation( EmuMode );
					if ( VIP )  {
						VIPLOCAL = FALSE;
						AUTOLF = FALSE;
						VTa = 0x02;
					}
					else
						VTa = 0x03;
					VideoHandler = Term[EmuMode].pointer;
					SendFunctionKey = Fkey[EmuMode].pointer;
					SendTab = Tkey[EmuMode].pointer;
					fsfill( VidRows, 1, 1, VidCols, ' ', LightGray+BlackBG );
					CursorOn();
					break;

				  case ALTR:			// Receive File

					CursorOff();
					TransferFile( "Receive", 'R' );
					gotoxy( VTc, VTr );
					CursorOn();
					break;

				  case ALTT:			// Transmit File

					CursorOff();
					TransferFile( "Send", 'T' );
					gotoxy( VTc, VTr );
					CursorOn();
					break;

				  case ALTH:			// Help ?

					CursorOff();
					switch( EmuMode )		{
						case 6:  GiveUserHelp( "VIP.HLP" );
							 break;
						default: GiveUserHelp( "T01.HLP" );
							 break;
					}
					CursorOn();
					break;

				  case ALTK:			// Kermit Send/Receive

					CursorOff();
					KermitCmds();
					CursorOn();
					break;

				  case ALTL:

					if ( VIP )  {
						VIPLOCAL = ( VIPLOCAL ) ? FALSE : TRUE;
						ECHOON = ( ECHOON ) ? FALSE : TRUE;
					}
					break;

				  case ALTQ:				// Quit Terminal

					if ( ONLINE )	{
						MakeWindow( 10, 18, 5, 45, LightCyan+BlueBG, LightCyan+BlueBG,
							Shadow, Frame1 );
						fswriteC( "Disconnect line first (Y/n) ? ", 12, VidCols, Yellow+BlueBG );
						ch = tolower( getch() );
						if ( ch == 'y' )	{
							fswrite( "Yes", 12, 55, Yellow+BlueBG );
							HangUpLine();
							CloseWindow();
						}
						else	{
							SHUTDOWN_SERIAL = FALSE;
							CloseWindow();
						}
					}

					QUIT = TRUE;
					break;

/*				  case ALTS:

					STATLINE = ( STATLINE ) ? FALSE : TRUE;
					break;	*/

				  case ALTX:  				// Hang-up Line.
											// Drop DTR or
					HangUpLine();			// Send hang-up string
					break;

				//	Alt & Ctl F-Keys.
				//      Macros will be sent out port

				  case AF1:
				  case AF2:
				  case AF3:
				  case AF4:
				  case AF5:
				  case AF6:
				  case AF7:
				  case AF8:
				  case AF9:
				  case AF10:

					strcpy( buff, MacroKeys.afkey[key-104] );

					while ( ( pp = strchr( buff, '|' ) ) != NULL )
						*pp = '\r';

					SerialString( buff );
					break;

				  case AF11:
				  case AF12:

					strcpy( buff, MacroKeys.afkey[key-129] );

					while ( ( pp = strchr( buff, '|' ) ) != NULL )
						*pp = '\r';

					SerialString( buff );
					break;

				  case CF1:
				  case CF2:
				  case CF3:
				  case CF4:
				  case CF5:
				  case CF6:
				  case CF7:
				  case CF8:
				  case CF9:
				  case CF10:

					strcpy( buff, MacroKeys.cfkey[key-94] );

					while ( ( pp = strchr( buff, '|' ) ) != NULL )
						*pp = '\r';

					if ( CTSOK )
						SerialString( buff );
					break;

				  case CF11:
				  case CF12:

					strcpy( buff, MacroKeys.cfkey[key-127] );
					while ( ( pp = strchr( buff, '|' ) ) != NULL )  *pp = '\r';
					SerialString( buff );
					break;

				  default:

					( * SendFunctionKey ) ( key );
					break;
				}
			}
			else	{
				if ( ( !VIPLOCAL ) && ( ch == TAB ) )
					( * SendTab ) ();

				else

				if ( !VIPLOCAL )
					SerialOut( ch );

				if ( VIPLOCAL )
					( * VideoHandler ) ( ch );

				else

				if ( ECHOON )    			// Check echo
					( * VideoHandler ) ( ch );
			}
		}

		else

		if ( ( c = getccb() ) != -1 )  {

		// Send the characater received through the current
		// terminal emulator.

			( * VideoHandler ) ( c );
		}

		// Screen SError from serial io interrupt

		switch ( SError ) {

			case NOERROR:

				break;

			case BUFOVFL:

				fprintf(stderr, "\nBuffer Overflow.\n");
				break;

			case CARRIERLOST:

				if ( ONLINE )	{
					MakeWindow( 10, 18, 5, 45, LightCyan+BlueBG, LightCyan+BlueBG,
						Shadow, Frame1 );
					fswriteC( "CARRIER LOST!", 12, VidCols, Yellow+BlueBG );
					ONLINE = FALSE;
					CARRIERON = FALSE;
					Beep( 200 );
					delay( 3000 );
					CloseWindow();
				}
				SError = NOERROR;

				break;

			default:

				fprintf(stderr, "\nUnknown Error, SError = %d\n", SError);
				break;
		}
		if ( VIP )			// Vip emulation status line display
			DisplayVipStatus();

		gotoxy( VTc, VTr );

	} while ( !QUIT );			// End of TerminalOps()

//	ONLINE = CARRIERON;
//	if ( ONLINE )
//		HangUpLine();
	if ( SHUTDOWN_SERIAL )
		closeserial();
	RestTermScreen();
	CursorOff();
	return;

} // end of TerminalOps()