/*
**  terminal.h (FrontDoor Terminal)
**
**  Copyright 1991 Joaquim H. Homrighausen. All rights reserved.
**
**  TERMPHON.FD, TERMPROF.FD, and TERMKEYS.FD format for FrontDoor 2.01+
**
**  Last revised:   91-10-02
**
**  -------------------------------------------------------------------------
**  This information is not necessarily final and is subject to change at any
**  given time without further notice
**  -------------------------------------------------------------------------
*/

    /* Phone entry record descriptor -------------------------------------- */
    /* TERMPHON.FD */

    /* Note that the char[] fields are in Pascal fashion. The first byte is
    the length byte. The actual string starts at [1] and the string is NOT
    NUL terminated. In Pascal they would be string[30] and string[34].*/

    typedef struct
        {
        char            name[31];                              /*System name*/
        char            number[35];             /*Telephone number as dialed*/
        unsigned int    baud;                                    /*Baud rate*/
        unsigned long   flags;              /*See below under 'Entry Flags:'*/
        unsigned char   protocol,         /*Preferred file transfer protocol*/
                        parity,                                     /*Parity*/
                        wordlen,                    /*# of data bits, 7 or 8*/
                        stopbits,                   /*# of stop bits, 1 or 2*/
                        emulation;                      /*Terminal emulation*/
        unsigned int    keyset;      /*Key definition record, 0xFFFF=DEFAULT*/
        unsigned int    profile;                 /*Profile record, 0=DEFAULT*/
        char            script[9];       /*Script name to use, blank if none*/
        }
    PHONEREC;

    /* Phone record flags: */

#define XLAT_ON         0x00000001L                 /*Use translation tables*/
#define DIALED          0x00000002L                     /*Queued for dialing*/
#define LINEFEEDS       0x00000004L                /*On=CR->CR/LF Off=CR->CR*/
#define NOWRAPLINES     0x00000008L          /*Don't wrap lines when xpos=80*/
#define B_LOCALECHO     0x00000010L       /*Echo keyboard input to local CRT*/
#define B_NOAVATAR      0x00000020L                    /*No AVATAR sequences*/
#define B_NOIEMSI       0x00000040L                               /*No IEMSI*/
#define DELETED         0x80000000L         /*Entry is deleted, never stored*/

    /* Keyset entry record descriptor ------------------------------------- */
    /* TERMKEYS.FD */

    /* Note that the char[] fields are in Pascal fashion. The first byte is
    the length byte. The actual string starts at [1] and the string is NOT
    NUL terminated. In Pascal they would be string[30].*/

    typedef struct
        {
        char    shiftkey[12][31];                             /*Shift F1-F12*/
        char    ctrlkey[12][31];                               /*Ctrl F1-F12*/
        }
    KEYREC;

/* end of file "terminal.h" */

