
/*
		---------------------------------------
		PowerTerm (c) 1992-94  Byron D Copeland
			   All Rights Reserved.
		---------------------------------------


		Module - termops.c

		Handles all Terminal Emulator operations.
*/



#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <string.h>
#include <dir.h>

#include "pckeys.h"
#include "fsvlib.h"

#define FALSE		0
#define TRUE		(!FALSE)
#define NOERROR		0
#define CARRIERLOST	100     	// Lost Carrier
#define BUFOVFL         1       // Buffer overflowed
#define MSR				6
#define DCD				0x80	// Carrier-Detect
#define CTS				0x10	// Clear-to-Send
#define ZSPLAT			'*'
#define ZCANZEL			24

FILE			*toCap;
char			*CapFilePath;
char			termCapName[13];
int			termCapOpen = FALSE;
int			CHATMODE = FALSE;

extern char	*buff;
extern char	*TempBuf;
extern char	*InitBuf;
extern char	*fileswork1;
extern char 	*EmuData1[8];
extern char	*PTEnviron;

extern int	FIFO;
extern int	UPPERONLY;
extern int	CARRIERON;
extern int     	ECHOON;
extern int      LFON;
extern int      ROLL;
extern int	VIP;
extern int      VIPLOCAL;
extern int	VIPHIDE;
extern int	INSMODE;
extern int  	AUTOLF;
extern int	KB_101;
extern int	portbase;
extern int	SError;
extern int	MacChk;
extern int	FnChk;
extern int  	VTr;
extern int	VTc;
extern int	VTwb;
extern int	VTwt;
extern int	VTa;
extern int	parity;
extern int	bits;
extern int	stopbits;
extern int	ParityData2[5];

extern unsigned EmuMode;
extern unsigned	OLDVMODE;
extern unsigned port;

extern unsigned long speed;
extern unsigned long savspeed;

extern struct MACROS  {
	char			*mname;		// 8 Bytes
	char			*mdesc;		// 60 Bytes
	char			*afkey[12];
	char			*cfkey[12];
} MacroKeys;


extern struct MODEMPARMS {
	unsigned long	defs[10];
	char			inits[11][36];
	char			modemmsg[15][16];
}  ModemParms;


extern struct {
	void ( *pointer ) ();
} Term[20];

extern struct {
	void ( *pointer ) ();
} Fkey[20];

extern struct {
	void ( *pointer ) ();
} Tkey[20];
extern FILE 	*toCap;
extern int	termCapOpen;

// Local function prototypes

void TerminalOps( void );
void termCaptureFile( void );
void updateStatusLine( void );
void displayDosError( int errornum );

/*-----------------------------------------------------------------------

	TerminalOps()	- function to handle all terminal emulations &
			  keyboard input.

------------------------------------------------------------------------*/

void TerminalOps( void )
{
	int		QUIT;
	int		c;
	int		ch;
	int		key;
	int     	CTSOK;
	int		gotZSPLAT1;
	int		gotZSPLAT2;
	int		gotZCAN;
	int		SHUTDOWN_SERIAL;
	int		ONLINE;
	char 		buf[ 32 ];
	int		i;
	char		*pp;
	void		( * VideoHandler ) ( int c );			// Emulation funtion pointer
	void		( * SendFunctionKey ) ( int key );		// Funtion key function pointer
	void		( * SendTab ) ();				// Tab key function pointer

	// Struct for macro keys

	union REGS toxr;

	gotZSPLAT1 = FALSE;
	gotZSPLAT2 = FALSE;
	INSMODE = FALSE;
	SHUTDOWN_SERIAL = TRUE;
	CTSOK = FALSE;
	LFON = FALSE;
	ROLL = TRUE;
	ECHOON = FALSE;
	VIP = FALSE;
	OLDVMODE = 0;
	VIPLOCAL = FALSE;
	ONLINE = FALSE;
	SError = NOERROR;
	VIPHIDE = FALSE;
	MacChk = FALSE;
	FnChk = FALSE;
	QUIT = FALSE;
	CHATMODE = FALSE;
	EmuMode = (unsigned int)ModemParms.defs[9];
	textcolor( 3 );
	setstrng( CapFilePath, '\0', MAXPATH );

	// Set up emulation pointers

	VideoHandler = Term[EmuMode].pointer;
	SendFunctionKey = Fkey[EmuMode].pointer;
	SendTab = Tkey[EmuMode].pointer;

	// Cursor at home

	VTr = 1;
	VTc = 1;

	VTwt = 1;				// Window Top
	VTwb = 24;				// Window Bottom

	SaveTermScreen();
	VTa = 0x03;
	fsfill( 1, 1, VidRows, VidCols, ' ', Cyan+BlackBG );
	CursorOn();
	gotoxy( VTc, VTr );

	// Start up with default port settings

	speed = ModemParms.defs[1];
	port = ( unsigned int ) ModemParms.defs[0];
	parity = ParityData2[( ( unsigned int ) ModemParms.defs[3] )];
	bits = ( int )ModemParms.defs[2];
	stopbits = ( int ) ModemParms.defs[4];
//	TimeOutV = ( int ) ModemParms.defs[6];

	// Initialize Serial with Default settings.

	FIFO = FALSE;
	savspeed = speed;
//	fswrite( "Setting Serial", VTr++, VTc, VTa);
	if ( SetSerial( port,
		( unsigned long )speed, parity, bits, stopbits ) != 0 )	{
		fprintf( stderr, "\n!!!! Commmunications Port Initialization Error !!!!!\n" );
		fprintf( stderr, "Please ensure that the com port exists.  Hit any key to continue...\n" );
		Bog();
		ch = getch();
		kbflush();
		RestTermScreen();
		CursorOff();
		return;
	}
	initserial();
//	flushinput();

	fswrite( "PowerTerm v0.04 (beta)  Copyright (c) 1992-94  Byron D Copeland", VTr++, VTc, VTa);
	if ( FIFO )
		fswrite( "16550AF Detected!", VTr++, VTc, VTa );
	else
		fswrite( "8250/16450 Detected!", VTr++, VTc, VTa );
	fsfill( VTr++, 1, 1, VidCols, '', VTa );
	gotoxy( VTc, VTr );
	updateStatusLine();

	// If we have something to talk to, then try to send the init
	// strings.

	CARRIERON = ( ( inportb( portbase + MSR ) & DCD ) == DCD ) ? TRUE : FALSE;
	ONLINE = CARRIERON;
	CTSOK = ( ( inportb( portbase + MSR ) & CTS ) == 0 ) ? FALSE : TRUE;
	if ( !CTSOK )	{
		MakeWindow( 10, 20, 4, 40, LightCyan+BlueBG, LightCyan+BlueBG,
			Shadow, Frame1 );
		fswriteC( "CTS is not active. Waiting...", 11, VidCols, Yellow+BlueBG );
		fswriteC( "Press any key to CANCEL wait.", 12, VidCols, Yellow+BlueBG );
		do	{
			CTSOK = ( ( inportb( portbase + MSR ) & CTS ) == 0 ) ? FALSE : TRUE;
		}	while( ( !CTSOK ) && ( !kbhit() ) );
		CloseWindow();
		kbflush();
	}
	if ( ( CTSOK ) && ( !ONLINE ) )	{
		fswrite( "Initializing...", VTr, VTc, VTa );
		gotoxy( 16, VTr );
		delay( 800 );
		setstrng( TempBuf, '\0', 50 );
		sprintf( TempBuf, "%s\0", ModemParms.inits[9] );
		if ( *TempBuf ) 	{
			while( ( pp = strstr( TempBuf, "|" ) ) != NULL )
				 *pp='\r';
			SerialString( TempBuf );

			// Throw out anything echoed!

			flushinput();
			delay( ModemParms.defs[5] * 100 );
			sprintf( TempBuf, "%s\0", ModemParms.modemmsg[0] );
			if ( *TempBuf ) 	{
				while( ( pp = strstr( TempBuf, "|" ) ) != NULL )
					*pp='\r';
				if ( !WaitForModemMsg( TempBuf, 10 ) )
					fswriteM( "x", VTa );
				else
					fswriteM( ".", VTa );
				gotoxy( 17, VTr );
			}
		}
	}
	else
	if ( ( CTSOK ) && ( ONLINE ) )	{
		fswrite( "Carrier-Detect!", VTr++, VTc, VTa );
		gotoxy( VTc, VTr );
	}
	flushinput();
	CTSOK = ( ( inportb( portbase + MSR ) & CTS ) == 0 ) ? FALSE : TRUE;
	if ( ( CTSOK ) && ( !ONLINE ) )	{
		setstrng( TempBuf, '\0', 50 );
		sprintf( TempBuf, "%s\0", ModemParms.inits[10] );
		if ( *TempBuf ) 	{
			while( ( pp = strstr( TempBuf, "|" ) ) != NULL )
				*pp='\r';
			SerialString( TempBuf );

			// Throw out anything echoed!

			flushinput();
			delay(ModemParms.defs[5] * 100);
			sprintf( TempBuf,"%s\0",ModemParms.modemmsg[0] );
			if ( *TempBuf ) 	{
				while( ( pp = strstr( TempBuf, "|" ) ) != NULL )
					*pp='\r';
				if ( !WaitForModemMsg( TempBuf, 10 ) )
					fswriteM( "x", VTa );
				else
					fswriteM( ".", VTa );
			}
		}
	}
	fswrite( "PowerTerm Ready!", ++VTr, VTc, VTa );
	VTr++;
	gotoxy( VTc, VTr );

	// Send the ATTENTION string.

	sprintf( buff, "%s\0", ModemParms.inits[4] );
	while( ( pp = strstr( buff, "|" ) ) != NULL )
		*pp = '\r';
	CTSOK = ( ( inportb( portbase + MSR ) & CTS ) == 0 ) ? FALSE : TRUE;
	if ( ( CTSOK ) && ( !ONLINE ) )    {
		if ( *buff )
			SerialString( buff );
	}

	//  Terminal Emulator Loop
	//  Loop until OODone = TRUE

	do	{

		// Poll Keyboard Buffer

		if ( KB_101 )
			toxr.h.ah = 0x11;
		else
			toxr.h.ah = 0x01;
		int86( 0x16, &toxr, &toxr );
		if ( !( toxr.x.flags & 0x40 ) )	{	// key hit?

			if ( ( VIP ) && ( KB_101 ) )	{
				if ( toxr.h.ah == 0xe0 )	{
					if ( toxr.h.al == 0x0d ) 	{	// Keypad Enter
						SerialString( "\033i\0" );
						kbflush();
						continue;
					}
				}
			}
			ch = getch();
			if ( ch == SPECIALKEY )  {
				switch( key = getch() )  {

				  case ALT_EQ:		// Alt + =

					if ( ( VIP ) && ( !KB_101 ) )
						SerialString( "\033i\0" );
					break;

				  case ALTA:			// VIP ONLY - Auto Linefeed

					if ( VIP )
						AUTOLF = ( AUTOLF ) ? FALSE : TRUE;
					else
						LFON = ( LFON ) ? FALSE : TRUE;
					break;

				  case ALTB:			// Macros

					CursorOff();
					EditMacroKeys();
					CursorOn();
					updateStatusLine();
					break;

				  case ALTC:			// Screen Capture on/off

					termCaptureFile();
					CursorOn();
					updateStatusLine();
					break;

				  case ALTD:			// Dialing Directory

					if ( !ONLINE )  {
						CursorOff();
						ONLINE = DisplayDialDirectory();
						fsfill( VidRows, 1, 1, VidCols, ' ', LightGray+BlackBG );
						if ( VIP )  {
							VIPLOCAL = FALSE;
							AUTOLF = FALSE;
						}
						VideoHandler = Term[EmuMode].pointer;
						if ( VIP )
							VTa = 0x02;
						else
							VTa = 0x03;
						SendFunctionKey = Fkey[EmuMode].pointer;
						SendTab = Tkey[EmuMode].pointer;
						CursorOn();
						updateStatusLine();
					}
					break;

				  case ALTE:			// Emulation

					CursorOff();
					EmuMode = SwitchEmulation( EmuMode );
					if ( VIP )  {
						VIPLOCAL = FALSE;
						AUTOLF = FALSE;
						VTa = 0x02;
					}
					else
						VTa = 0x03;
					VideoHandler = Term[EmuMode].pointer;
					SendFunctionKey = Fkey[EmuMode].pointer;
					SendTab = Tkey[EmuMode].pointer;
					updateStatusLine();
					CursorOn();
					break;

				  case ALTR:			// Receive File

					if ( CHATMODE )
						break;
					CursorOff();
					TransferFile( "Receive", 'R' );
					CursorOn();
					break;

				  case ALTT:			// Transmit File

					if ( CHATMODE )
						break;
					CursorOff();
					TransferFile( "Send", 'T' );
					CursorOn();
					break;

				  case ALTH:			// Help ?

					CursorOff();
					switch( EmuMode )		{

						case 6:

							GiveUserHelp( "VIP.HLP" );
							break;

						default:

							GiveUserHelp( "T01.HLP" );
							break;
					}
					CursorOn();
					break;

				  case ALTK:			// Kermit Send/Receive

					if ( CHATMODE )
						break;
					CursorOff();
					KermitCmds();
					CursorOn();
					break;

				  case ALTL:			// VIP ONLY - Local Mode on/off

					if ( VIP )  {
//                                        	NECHO = FALSE;
						VIPLOCAL = ( VIPLOCAL ) ? FALSE : TRUE;
						ECHOON = ( ECHOON ) ? FALSE : TRUE;
					}
					break;

				  case ALTP:			// Push to DOS

					dosShell();
					CursorOn();
					break;

				  case ALTQ:				// Quit Terminal

					if ( ONLINE )	{
						MakeWindow( 10, 18, 5, 45, LightCyan+BlueBG, LightCyan+BlueBG,
							Shadow, Frame1 );
						fswriteC( "Disconnect line first (Y/n) ? ", 12, VidCols, Yellow+BlueBG );
						ch = tolower( getch() );
						if ( ( ch == 'y' ) || ( ch == CR ) )	{
							fswrite( "Yes", 12, 55, Yellow+BlueBG );
							HangUpLine();
							CloseWindow();
						}
						else	{
							SHUTDOWN_SERIAL = FALSE;
							CloseWindow();
						}
					}
					kbflush();
					QUIT = TRUE;
					break;

				  case ALTW:

					if ( VIP )
						break;
					CHATMODE = ( CHATMODE ) ? FALSE : TRUE;
					updateStatusLine();
					break;

				  case ALTX:  				// Hang-up Line.
											// Drop DTR or
					HangUpLine();			// Send hang-up string
					updateStatusLine();
					break;

				//	Alt & Ctl F-Keys.
				//      Macros will be sent out port

				  case AF1:
				  case AF2:
				  case AF3:
				  case AF4:
				  case AF5:
				  case AF6:
				  case AF7:
				  case AF8:
				  case AF9:
				  case AF10:

					strcpy( buff, MacroKeys.afkey[key-104] );
					while ( ( pp = strchr( buff, '|' ) ) != NULL )
						*pp = '\r';
					SerialString( buff );
					break;

				  case AF11:
				  case AF12:

					strcpy( buff, MacroKeys.afkey[key-129] );
					while ( ( pp = strchr( buff, '|' ) ) != NULL )
						*pp = '\r';
					SerialString( buff );
					break;

				  case CF1:
				  case CF2:
				  case CF3:
				  case CF4:
				  case CF5:
				  case CF6:
				  case CF7:
				  case CF8:
				  case CF9:
				  case CF10:

					strcpy( buff, MacroKeys.cfkey[key-94] );
					while ( ( pp = strchr( buff, '|' ) ) != NULL )
						*pp = '\r';
					if ( CTSOK )
						SerialString( buff );
					break;

				  case CF11:
				  case CF12:

					strcpy( buff, MacroKeys.cfkey[key-127] );
					while ( ( pp = strchr( buff, '|' ) ) != NULL )  *pp = '\r';
					SerialString( buff );
					break;

				  default:

					( * SendFunctionKey ) ( key );
					break;
				}
			}
			else	{
				if ( ( !VIPLOCAL ) && ( ch == TAB ) )
					( * SendTab ) ();
				else
				if ( !VIPLOCAL )
					while ( SerialOut( ch ) == -1 );
				if ( VIPLOCAL )
					( * VideoHandler ) ( ch );
				else
				if ( ECHOON )		// Check echo
					( * VideoHandler ) ( ch );
			}
		}
		else
		while ( ( c = getccb() ) != -1 )  {

		// Send the characater received through the selected
		// terminal emulator.

/*			if ( c == '*' )	{

				i = 0;
				setstrng( buf, '\0', 31 );

				buf[ i ] = c;
				while( ( ( c = getccb() ) != -1 ) && ( i < 30 ) )
					buf[++i] = c;
				if ( strstr( buf, "**\x18" ) )	{
					for ( i = 0; buf[ i ]; i++ )
						ScreenCharOut( buf[ i ] );
					flushinput();
					zmReceive();
				}
				else	{
					for ( i = 0; buf[ i ]; i++ )
						( *VideoHandler ) ( buf[ i ] );
				}
			}
			else*/
				( *VideoHandler ) ( c );
			if ( ( CHATMODE ) && ( ONLINE ) && ( !VIP ) )	{
				while( SerialOut( c ) == -1);
				if ( c == '\r' )
					( *VideoHandler ) ( '\n' );
			}
//			if ( termCapOpen )				// Check for capture
//				fputc( c, toCap );
			if ( ( c == '\r' ) && ( LFON ) )		// CR Expansion
				fputc( 0x0a, toCap );
		}

		// Screen SError from serial io interrupt

		switch ( SError ) {

			case NOERROR:

				break;

			case BUFOVFL:

				fprintf(stderr, "\nBuffer Overflow.\n");
				break;

			case CARRIERLOST:

				if ( ONLINE )	{
					CursorOff();
					MakeWindow( 10, 18, 5, 45, LightCyan+BlueBG, LightCyan+BlueBG,
						Shadow, Frame1 );
					fswriteC( "CARRIER LOST!", 12, VidCols, Yellow+BlueBG );
					ONLINE = FALSE;
					CARRIERON = FALSE;
//					Beep( 200 );
					Bog();
					delay( 1000 );
					CloseWindow();
					CursorOn();
				}
				SError = NOERROR;

				break;

			default:

				fprintf(stderr, "\nUnknown Error, SError = %d\n", SError);
				break;
		}
		if ( VIP )	// Vip emulation status line display
			DisplayVipStatus();
		gotoxy( VTc, VTr );
	} while ( !QUIT );			// End of TerminalOps()

	// Do we want to turn off the serial port?

	if ( SHUTDOWN_SERIAL )	{
		closeserial();
		if ( termCapOpen )	{	// Close capture file ?
			fclose( toCap );
			termCapOpen = FALSE;
		}
	}
	RestTermScreen();
	CursorOff();
	return;

} // end of TerminalOps()


/*-------------------------------------------------------------------------

 -------------------------------------------------------------------------*/

void termCaptureFile( void )
{
	int flag;
	char	drive[MAXDRIVE];
	char	dir[MAXDIR];
	char	filename[MAXFILE];
	char	ext[MAXEXT];

	struct DOSERROR dosErrorInfo;

	if ( termCapOpen )	{
		fclose( toCap );
		setstrng( termCapName, '\0', 12 );
		termCapOpen = FALSE;
		return;
	}

	if (!*CapFilePath )
		strcpy( CapFilePath, strupr( PTEnviron ));

	MakeWindow( 15, 2, 5, 77, LightCyan+BlueBG, LightCyan+BlueBG,
			Shadow, Frame1 );
	fswriteC( " Open Capture File ", 15, VidCols, LightCyan+BlueBG );
	fsattr( 15, 30, 1, 19, Blue+CyanBG );
	fswrite( "Path: ", 17, 4, LightCyan+BlueBG );
	fswrite(" Press <", 19, 52, LightCyan+BlueBG );
	fswriteM( "Enter", Yellow+BlueBG );
	fswriteM( "> when done ", LightCyan+BlueBG );
	flag = FALSE;
	do	{
		setstrng( InitBuf, '', MAXDIR );
 //		setstrng( CapFilePath, '\0', MAXDIR );

		FnChk = TRUE;
		UPPERONLY = TRUE;
		EditStringValue( CapFilePath, MAXDIR, 17, 10, InitBuf,
			Cyan+BlueBG, Yellow+CyanBG, Yellow+BlueBG );
		UPPERONLY = FALSE;
		FnChk = FALSE;
		if ( !*CapFilePath )	{
			break;
		}
		if ( ( toCap = fopen( CapFilePath, "w" ) ) == NULL )	{
//		toCap = fopen( CapFilePath, "w" );

			if ( dosexterr( &dosErrorInfo ) )
				displayDosError( dosErrorInfo.de_exterror );
		}
		else	{
			termCapOpen = TRUE;
			flag = TRUE;
		}
	}	while( !flag );
	CloseWindow();
	fnsplit( CapFilePath, drive, dir, filename, ext );
	sprintf( termCapName, "%s%s", filename, ext );
	return;
}

/*-------------------------------------------------------------------------

 -------------------------------------------------------------------------*/


void updateStatusLine( void )
{
	fsfill( 25, 1, 1, VidCols, ' ', Blue+CyanBG );
	fswrite( "Alt+H Help ", 25, 2, LastAttr );
	fswriteM( EmuData1[EmuMode], LastAttr );
	fswriteM( "", LastAttr );
	sprintf( TempBuf, " %lu ", speed );
	fswriteM( TempBuf, LastAttr );
	if ( CHATMODE )
		fswriteM( " CHATMODE ", LastAttr );
	fswrite( " Capture: ", 25, 57, LastAttr );
	if ( *termCapName )
		fswriteM( termCapName, LastAttr );
	else
		fswriteM( "Closed      ", LastAttr );
}



/*-------------------------------------------------------------------------

			PowerTerm v0.01 (c) 1992-1993 Byron D Copeland

 -------------------------------------------------------------------------*/