
/*
		---------------------------------------
		PowerTerm (c) 1992-93  Byron D Copeland
			 All Rights Reserved.
		---------------------------------------

*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <dos.h>

#define FALSE	0
#define TRUE	(!FALSE)
#define	timer	0x1c

int	TIME_OUT;
int	KEY_HIT;

/*-------------------------------------------------------------------------

	Portions of these timer functions written by John Junod.

	Decsription of Timer Functions:

	alarm() - Provides a timeout function for number of
		  seconds required.  Used mainly for waiting
		  for a character to arrive at the serial port.

	Since DOS doesn't have a SIGALRM we came up with a
	decent alternative.  I used SIGTERM instead.  What the
	hell, it works.

	Once alarm() is set up.  char_waiting is used to raise
	SIGTERM if a timeout occurs when no characters are placed
	into the comm buffer.

	Example:

	#include <setjmp.h>
	#include <signal.h>

	jmp_buf	xenv;

	void My_Process( int tmint )
	{
		if ( setjmp( xenv ) )	{	// Return here if timeout

			// Notify user of this timeout

			alarm( 0 );
			return( FALSE );
		}

		if ( signal( SIGTERM, jump_back ) == SIG_ERR )	{

			// Notify user of fatal error

			alarm( 0 );
			return( FALSE );
		}

		alarm( tmint );			// Set timer on.

		// ...     Do something.
		// ...	   Do something.
		// ...	   Do something.

		alarm( 0 );			// Turn off timer now.
	}

	  This function is entered whenever SIGTERM is raised.
	  It then returns TRUE.

	void jump_back(int sig)
	{
		if ( sig == SIGTERM )
			longjmp( xenv, TRUE);	//  Give up.
	}

 *-----------------------------------------------------------------------*/

static int 	timeronflag = 0;
int		timeleft = 0;
static int	eighteenth = 0;

int char_waiting( void );
void alarm( int secs );
void interrupt far ( *oldtimer )( void );		/* Save the original timer interrupt    */
void interrupt far mytimer( void );


/*
 *	char_waiting() - written by Byron Copeland
 *
 *	Loop until a character arrives in the receive buffer
 *	or until no timeleft to wait.
 */

int char_waiting()
{
	int 	c;

	do  {
		if ( fast_kbhit() )	{
			KEY_HIT = TRUE;
			return( -1 );
		}
		else
		if ( timeleft < 1 )
			raise( SIGTERM );
	} while (( c = getccb() ) == -1 );
	return( c );
}


/*
 *
 */

void alarm( int secs )
{
	if ( secs == 0 )  {			// Yes? shut it down
		if ( timeronflag == 1 )  {
			disable();
			setvect( timer, oldtimer );
			enable();
			timeronflag = 0;
		}
		return;
	}
	eighteenth = 0;
	timeleft = secs;
	if ( timeronflag == 0 )  {		// Is it already running?
		oldtimer = getvect( timer );
		disable();
		setvect( timer, mytimer );
		enable();
		timeronflag = 1;
	}
	return;
}

/*-----------------------------------------------------------------------
 *  Function mytimer will be executed 18.2 times a second when
 *  installed by alarm().
 *---------------------------------------------------------------------*/

void interrupt far mytimer( void )
{
	if ( ++eighteenth > 17 )  {
		--timeleft;
		eighteenth = 0;
	}
	oldtimer();
}





