
/*-----------------------------------------------------------------------
   Module VIDEOLIB.C - Video Library to handle all display functions.

				Copyright (c) 1991-93  Byron D Copeland
						All Rights Reserved.

------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <alloc.h>
#include <mem.h>
#include <dos.h>
#include "videolib.h"

/********************************************************************/

int Init_FSVLib ()
{
  union REGS xregs;

  xregs.h.ah = 0x0f;
  int86(0x10,&xregs,&xregs);
  if (xregs.h.al == 7)
	VideoSeg = 0xb000;
  else
	VideoSeg = 0xb800;
  VidRows = 25;
  VidCols = 80;
  Shadow = 1;
  NoShadow ^= NoShadow;
  WINLOOK = FALSE;
  VLastPos ^= VLastPos;
  LastAttr = 0x07;

/*  Set up our border structures.                                    */

  strcpy (Brdr0.TBL," ");
  strcpy (Brdr0.LRS," ");
  strcpy (Brdr0.TLC," ");
  strcpy (Brdr0.TRC," ");
  strcpy (Brdr0.BLC," ");
  strcpy (Brdr0.BRC," ");

  strcpy (Brdr1.TBL,"");
  strcpy (Brdr1.LRS,"");
  strcpy (Brdr1.TLC,"");
  strcpy (Brdr1.TRC,"");
  strcpy (Brdr1.BLC,"");
  strcpy (Brdr1.BRC,"");

  strcpy (Brdr2.TBL,"");
  strcpy (Brdr2.LRS,"");
  strcpy (Brdr2.TLC,"");
  strcpy (Brdr2.TRC,"");
  strcpy (Brdr2.BLC,"");
  strcpy (Brdr2.BRC,"");

/*   Lets re-map the default colors.                                 */

  Black = BLACK;
  Blue = BLUE;
  Green = GREEN;
  Red = RED;
  Brown = BROWN;
  Magenta = MAGENTA;
  Cyan = CYAN;
  Gray = DARKGRAY;

  LightGray = LIGHTGRAY;
  LightBlue = LIGHTBLUE;
  LightGreen = LIGHTGREEN;
  LightRed = LIGHTRED;
  Yellow = YELLOW;
  LightMagenta = LIGHTMAGENTA;
  LightCyan = LIGHTCYAN;
  White = WHITE;

  BlackBG = ( BLACK << 4 );
  BlueBG = ( BLUE << 4 );
  GreenBG = ( GREEN << 4 );
  RedBG = ( RED << 4 );
  BrownBG = ( BROWN << 4 );
  MagentaBG = ( MAGENTA << 4 );
  CyanBG = ( CYAN << 4 );
  LightGrayBG = ( LIGHTGRAY << 4 );
  Blink = BLINK;
  CurrentWindow ^= CurrentWindow;
  Frame0 = 0;
  Frame1 = 1;
  Frame2 = 2;

  /*  Swipe some memory for original screen.		*/

  if ((ScrnSave=(char far *)farmalloc (4000L)) == NULL)  {
     return (1);
  }
  SaveScreen ();
  return (0);
}


void fswrite( char *strng, unsigned int row,
		  unsigned int column, unsigned int attribute )
{
	static char *s;
	static unsigned int col;

	col = column;
	s = strng;
	while ( *s )  {
		fswchar( *s++, row, col, attribute );
		col++;
	}
	VLastPos = ( ( ( ( row - 1 ) * VidCols ) + ( col - 1 ) ) << 1 );
	LastAttr = attribute;
}


void fswchar( char schar, unsigned int row,
		  unsigned int column, unsigned int attribute )
{
	static unsigned int col;

	col = column;
	pokeb( VideoSeg,( ( ( ( row - 1 ) * VidCols ) + ( col - 1 ) ) << 1 ), schar );
	pokeb( VideoSeg,( ( ( ( row - 1 ) * VidCols ) + ( col - 1 ) ) << 1 ) + 1, attribute );
	VLastPos = ( ( ( ( row - 1 ) * VidCols ) + ( col - 1 ) ) << 1 );
	LastAttr = attribute;
}



void fswriteM (char *strng, unsigned int attribute)
{
	static char *s;

	s = strng;
	while ( *s )  {
		pokeb( VideoSeg, VLastPos, *s++ );
		pokeb( VideoSeg, VLastPos + 1, attribute );
		VLastPos++;
		VLastPos++;
	}
	LastAttr = attribute;
}


void fswriteC (char *strng, unsigned int row, unsigned int column,
				unsigned int attribute)
{
	static unsigned int col;
	static char *s;

	s=strng;
	if (*s)  {
		col = ((column - strlen(strng)) >> 1 );

		while (*s)  {
			fswchar(*s++,row,col,attribute);
			col++;
		}
		VLastPos = ( ( ( ( row - 1 ) * VidCols ) + ( col - 1 ) ) << 1 );
	}
	LastAttr = attribute;
}


/*********************************************************************/
/*  Function to fill the screen display with a character.            */
/*********************************************************************/

void fsfill 	(unsigned int row, unsigned int column,
		unsigned int rows, unsigned int columns,
		int fchar, unsigned int attribute)
{
	unsigned int 	coln;
	unsigned int 	rown;

	for ( rown = row; rown < ( row + rows ); rown++ )  {
		coln = column;

		while ( coln < ( column + columns ) )  {
			fswchar( fchar, rown, coln, attribute );
			coln++;
		}
	}
	VLastPos = ( ( ( ( rown - 2 ) * VidCols ) + ( coln - 1 ) ) << 1 );
	LastAttr = attribute;
}


/*********************************************************************/
/*  Function to fill the screen display with a attribute.            */
/*********************************************************************/

void fsattr (unsigned int row, unsigned int column,
		 unsigned int rows, unsigned int columns,
		 unsigned int attribute)
{
	unsigned int 	coln;
	unsigned int	rown;

	for ( rown = row; rown < ( row + rows ); rown++ )  {
		coln = column;

		while ( coln < ( column + columns ) )  {
			pokeb( VideoSeg,( ( ( ( rown - 1 ) * VidCols ) + ( coln - 1 ) ) << 1 ) + 1, attribute );
			coln++;
		}
	}
	VLastPos = ( ( ( ( rown - 1 ) * VidCols ) + ( coln - 1 ) ) << 1 );
	LastAttr = attribute;
}

/**********************************************************************/
/*  Function MakeWindow() will allocate a window structure, then      */
/*  allocate enough memory to store the present coordinates.  This    */
/*  will then call Boxit() to frame the window.                       */
/*  This function return 1 if successful, return 0 if not.            */
/**********************************************************************/

int MakeWindow (unsigned int row, unsigned int column,
		    unsigned int rows, unsigned int columns,
		    unsigned int attr1, unsigned int attr2,
		    unsigned int Shadow, unsigned int BType)

{

	static unsigned int 	ShadowR;
	static unsigned int		ShadowC;
	static unsigned int		ncols;
	static unsigned int		nrows;
	static long				winSpace;



/*  Make sure we are not over doing it!  */

	if ( CurrentWindow > 999 )
		return (0);

	if ( ( WindRec[CurrentWindow] =
		( struct Wndw far * ) farmalloc( sizeof( struct Wndw ) ) ) == NULL )
		return (0);


	if ((row > VidRows) || (row < 1))
		row = 1;

	if ( ( column > VidCols ) || ( column < 1 ) )
		column = 1;

	if ( ( row + rows ) > VidRows )  {
		rows = ( VidRows - row ) + 1;
		nrows = ( VidRows - row ) + 1;
		ShadowR = rows - 1;
	}
	else
	if ( ( row + rows ) == VidRows )  {
		rows = ( VidRows - row );
		nrows = ( VidRows - row );
		ShadowR = rows;
	}
	else {
		ShadowR = rows;
		nrows = row+rows;
	}

	if ( ( column + columns ) > VidCols )  {
		columns = ( VidCols - column ) + 1;
		ncols = ( VidCols - column ) + 1;
		ShadowC = columns - 2;
	}
	else
	if ( ( column + columns ) == VidCols )  {
		columns = ( VidCols - column );
		ncols = ( VidCols - column );
		ShadowC = columns - 1;
	}
	else {
		ShadowC = columns;
		ncols = column + columns;
	}


	WindRec[CurrentWindow]->Row=row;
	WindRec[CurrentWindow]->Col=column;
//  WindRec[CurrentWindow]->Row=1;
//  WindRec[CurrentWindow]->Col=1;
//  WindRec[CurrentWindow]->Rows=VidRows;
//  WindRec[CurrentWindow]->Cols=VidCols;
	if ( Shadow )	{
		WindRec[CurrentWindow]->Rows=row+ShadowR+1;
		WindRec[CurrentWindow]->Cols=column+ShadowC+1;
	}
	else	{
		WindRec[CurrentWindow]->Rows=nrows;
		WindRec[CurrentWindow]->Cols=ncols;
	}

	// Calculate window space.

	winSpace = ( ( ( long )WindRec[CurrentWindow]->Rows *
				   ( long )WindRec[CurrentWindow]->Cols ) << 1 );

	// Allocate window space.

	if ( ( WindRec[CurrentWindow]->LastScrn =
		( char far * ) farmalloc( winSpace ) ) == NULL )
		return (0);

	gettext( WindRec[CurrentWindow]->Col,
			 WindRec[CurrentWindow]->Row,
			 WindRec[CurrentWindow]->Cols,
			 WindRec[CurrentWindow]->Rows,
			 WindRec[CurrentWindow]->LastScrn );

//	gettext( column, row, ncols, nrows,	WindRec[CurrentWindow]->LastScrn );

/*  gettext (1, 1, VidCols, VidRows,
    WindRec[CurrentWindow]->LastScrn); */

	if ( Shadow )  {
		fsattr( row + 1, column + 2, ShadowR, ShadowC, Gray+BlackBG );
	}
	fsfill( row, column, rows, columns, ' ', attr1 );
	Boxit( row, column, rows, columns, attr2, BType );
	CurrentWindow += 1;
	return( 1 );
}


/**********************************************************************/
/*  Function ExplodeWindow() will allocate a window structure, then      */
/*  allocate enough memory to store the present coordinates.  This    */
/*  will then call Boxit() to frame the window.                       */
/*  This function return 1 if successful, return 0 if not.            */
/**********************************************************************/

int ExplodeWindow (unsigned int row, unsigned int column,
			unsigned int rows, unsigned int columns,
			unsigned int attr1, unsigned int attr2,
			unsigned int Shadow, unsigned int BType)

{

	static unsigned int	ShadowR;
	static unsigned int	ShadowC;
	static unsigned int	ncols;
	static unsigned int	nrows;
	static unsigned int	i;
	static unsigned int	j;
	static unsigned int	k;
	static unsigned int	l;
	static unsigned int	strow;
	static unsigned int	stcol;
	static unsigned int	toprow;
	static unsigned int	botrow;
	static unsigned int	numcols;
	static unsigned int	leftcol;
	static long			winSpace;



/*  Make sure we are not over doing it!  */

	if ( CurrentWindow > 999 )
		return (0);

	if ( ( WindRec[CurrentWindow] =
		( struct Wndw far * ) farmalloc( sizeof( struct Wndw ) ) ) == NULL )
		return (0);


	if ((row > VidRows) || (row < 1))
		row = 1;

	if ( ( column > VidCols ) || ( column < 1 ) )
		column = 1;

	if ( ( row + rows ) > VidRows )  {
		rows = ( VidRows - row ) + 1;
		nrows = ( VidRows - row ) + 1;
		ShadowR = rows - 1;
	}
	else
	if ( ( row + rows ) == VidRows )  {
		rows = ( VidRows - row );
		nrows = ( VidRows - row );
		ShadowR = rows;
	}
	else {
		ShadowR = rows;
		nrows = row+rows;
	}

	if ( ( column + columns ) > VidCols )  {
		columns = ( VidCols - column ) + 1;
		ncols = ( VidCols - column ) + 1;
		ShadowC = columns - 2;
	}
	else
	if ( ( column + columns ) == VidCols )  {
		columns = ( VidCols - column );
		ncols = ( VidCols - column );
		ShadowC = columns - 1;
	}
	else {
		ShadowC = columns;
		ncols = column + columns;
	}


	WindRec[CurrentWindow]->Row=row;
	WindRec[CurrentWindow]->Col=column;
	if ( Shadow )	{
		WindRec[CurrentWindow]->Rows=row+ShadowR+1;
		WindRec[CurrentWindow]->Cols=column+ShadowC+1;
	}
	else	{
		WindRec[CurrentWindow]->Rows=nrows;
		WindRec[CurrentWindow]->Cols=ncols;
	}

	// Calculate window space.

	winSpace = ( ( ( long )WindRec[CurrentWindow]->Rows *
				   ( long )WindRec[CurrentWindow]->Cols ) << 1 );

	// Allocate window space.

	if ( ( WindRec[CurrentWindow]->LastScrn =
		( char far * ) farmalloc( winSpace ) ) == NULL )
		return (0);

	gettext( WindRec[CurrentWindow]->Col,
			 WindRec[CurrentWindow]->Row,
			 WindRec[CurrentWindow]->Cols,
			 WindRec[CurrentWindow]->Rows,
			 WindRec[CurrentWindow]->LastScrn );

	strow = ( WindRec[CurrentWindow]->Row + WindRec[CurrentWindow]->Rows ) / 2;

	stcol = ( WindRec[CurrentWindow]->Col + WindRec[CurrentWindow]->Cols ) / 2;

	toprow = WindRec[CurrentWindow]->Row;
	leftcol = WindRec[CurrentWindow]->Col;

	botrow = WindRec[CurrentWindow]->Row + WindRec[CurrentWindow]->Rows;

	numcols = WindRec[CurrentWindow]->Cols / 2;


	if ( ( WindRec[CurrentWindow]->Row + WindRec[CurrentWindow]->Rows ) % 2 )
		k = 1;
	else							// is it odd or even?
		k = 0;

//	if ( ( WindRec[CurrentWindow]->Col + WindRec[CurrentWindow]->Cols ) % 2 )
//		l = 0;
//	else							// is it odd or even?
//		l = 1;

	l = 1;

	for ( i = strow, j = stcol ; i >= toprow; i--, j--, k+=2, l+=2 )	{
			if ( Shadow )
				fsattr( i + 1, j + 2, k, l, Gray+BlackBG );
			fsfill( i, j, k, l, ' ', attr1 );
			Boxit( i, j, k, l, attr2, BType );
	}
	k-=2;
	i++;
	l-=2;
	if ( j > leftcol )	{
		for ( ; j >= leftcol; j--, l+=2 )	{
			if ( Shadow )
				fsattr( i + 1, j + 2, k, l, Gray+BlackBG );
			fsfill( i, j, k, l, ' ', attr1 );
			Boxit( i, j, k, l, attr2, BType );
		}
	}
	CurrentWindow += 1;
	return( 1 );
}


/*********************************************************************/
/*  Function CloseWindow() will close the present window and         */
/*  de-allocate the space it is using.                               */
/*********************************************************************/

void CloseWindow ()
{
  CurrentWindow -= 1;
  if (CurrentWindow < 0)  {
    printf ("Window Error < 0\n");
    return;
  }
  puttext (WindRec[CurrentWindow]->Col,
	    WindRec[CurrentWindow]->Row,
	    WindRec[CurrentWindow]->Cols,
	    WindRec[CurrentWindow]->Rows,
	    WindRec[CurrentWindow]->LastScrn);
  farfree (WindRec[CurrentWindow]->LastScrn);
  farfree (WindRec[CurrentWindow]);

}



/**********************************************************************/
/*  Save entire screen until were done.                               */
/**********************************************************************/

void SaveScreen ()
{
  gettext (1, 1, VidCols, VidRows, (char far *)ScrnSave);
}

/**********************************************************************/
/*  Restore the original screen.                                      */
/**********************************************************************/

void RestoreScreen ()
{
  puttext (1, 1, VidCols, VidRows, (char far *)ScrnSave);
}

/**********************************************************************/
/*  Frame the windows we create.                                      */
/**********************************************************************/

void Boxit( unsigned int row, unsigned int column,
		unsigned int rows, unsigned int columns,
		unsigned int attribute, unsigned int BType )
{

	unsigned int	boxi;

	for ( boxi = column; boxi <= ( column + columns - 1 ); boxi++ )  {
		if ( BType == 0 )	{
			fswrite( Brdr0.TBL, row, boxi, attribute );
			fswrite( Brdr0.TBL, ( row + rows - 1 ), boxi, attribute );
		}
		else
		if ( BType == 1 )	{
			if ( WINLOOK )	{
				fswrite( Brdr1.TBL, row, boxi,
					( attribute & 0xF0 ) | LightGray );
				fswrite ( Brdr1.TBL, ( row + rows - 1 ), boxi,
					( attribute & 0xF0 ) | Gray );
			}
			else	{
				fswrite( Brdr1.TBL, row, boxi, attribute );
				fswrite( Brdr1.TBL, ( row + rows - 1 ), boxi, attribute );
			}
		}
		else	{
			fswrite( Brdr2.TBL, row, boxi, attribute );
			fswrite( Brdr2.TBL, ( row + rows - 1 ), boxi, attribute );
		}
	}
	for ( boxi = row; boxi <= ( row + rows - 1 ); boxi++ )  {
		if ( BType == 0 )  {
			fswrite( Brdr0.LRS, boxi, column, attribute );
			fswrite( Brdr0.LRS, boxi, ( column + columns - 1 ), attribute );
		}
		else
		if ( BType == 1 )  {
			if (WINLOOK)	{
				fswrite( Brdr1.LRS, boxi, column,
					( attribute & 0xF0 ) | LightGray );
				fswrite( Brdr1.LRS, boxi, ( column + columns - 1 ),
					( attribute & 0xF0 ) | Gray );
			}
			else  {
				fswrite (Brdr1.LRS, boxi, column, attribute);
				fswrite (Brdr1.LRS, boxi, (column+columns-1), attribute);
			}
		}
		else  {
			fswrite (Brdr2.LRS, boxi, column, attribute);
			fswrite (Brdr2.LRS, boxi, (column+columns-1), attribute);
		}
	}
	if (BType == 0)  {
		fswrite (Brdr0.TLC, row, column, attribute);
		fswrite (Brdr0.TRC, row, (column+columns-1), attribute);
		fswrite (Brdr0.BLC, (row+rows-1), column, attribute);
		fswrite (Brdr0.BRC, (row+rows-1), (column+columns-1), attribute);
	}
	else
	if (BType==1)  {
		if (WINLOOK)  {
			fswrite (Brdr1.TLC, row, column,
				(attribute & 0xF0) | LightGray);
			fswrite (Brdr1.TRC, row, (column+columns-1),
				(attribute & 0xF0) | Gray);
			fswrite (Brdr1.BLC, (row+rows-1), column,
				(attribute & 0xF0) | LightGray);
			fswrite (Brdr1.BRC, (row+rows-1), (column+columns-1),
				(attribute & 0xF0) | Gray);
		}
		else  {
			fswrite (Brdr1.TLC, row, column, attribute);
			fswrite (Brdr1.TRC, row, (column+columns-1), attribute);
			fswrite (Brdr1.BLC, (row+rows-1), column, attribute);
			fswrite (Brdr1.BRC, (row+rows-1), (column+columns-1), attribute);
		}
	}
	else  {
		fswrite (Brdr2.TLC, row, column, attribute);
		fswrite (Brdr2.TRC, row, (column+columns-1), attribute);
		fswrite (Brdr2.BLC, (row+rows-1), column, attribute);
		fswrite (Brdr2.BRC, (row+rows-1), (column+columns-1), attribute);
	}
}


void MakeButton( char *str, unsigned int Row, unsigned int Col,
		 unsigned int atr1, unsigned int atr2 )
{
	int 	len;

	len = strlen( str );
	fsfill( ( Row + 1 ), ( Col + 1 ), 1, len, '', atr2 );
	fswrite( str, Row, Col, atr1 );
	fswrite( "", Row, ( Col + len ), atr2 );
}

void CursorOff ()
{
	union REGS xregs;

	xregs.h.bh ^= xregs.h.bh;
	xregs.h.ah = 0x03;
	int86 (0x10, &xregs, &xregs);
	xregs.h.ch |= 0x20;
	xregs.h.ah = 0x01;
	int86 (0x10, &xregs, &xregs);
}

void CursorOn ()
{
	union REGS xregs;

	xregs.h.bh ^= xregs.h.bh;
	xregs.h.ah = 0x03;
	int86 (0x10, &xregs, &xregs);
	xregs.h.ch &= 0x5f;
	xregs.h.ah = 0x01;
	int86 (0x10, &xregs, &xregs);
}


/*********************************************************************/
/*  Th-Th-Thats all Folks!                                           */
/*********************************************************************/



